import os
import re
import time
from textwrap import wrap

import matplotlib.pyplot as plt
import numpy as np
import pathos.multiprocessing as mp
import plotly.graph_objs as plotly_graph
import seaborn as sns
from plotly.offline import plot, iplot, init_notebook_mode

import magine.html_templates.html_tools as ht
from magine.data.tools import log2_normalize_df

fold_change = 'fold_change'
flag = 'significant'
exp_method = 'source'
p_val = 'p_value'
rna = 'rna_seq'
gene = 'gene'
protein = 'protein'
metabolites = 'metabolites'
species_type = 'species_type'
sample_id = 'sample_id'
identifier = 'identifier'
label_col = 'label'

cm = plt.get_cmap('jet')


def write_table_to_html(data, save_name='index', out_dir=None,
                        run_parallel=False, exp_data=None,
                        plot_type='matplotlib'):
    """
    Creates a html table  of plots of genes for each ontology term.

    Parameters
    ----------
    data : magine.enrichment.enrichment_result.EnrichmentResult
    save_name : str
        name of html output file
    out_dir : str, optional
        output path for all plots
    run_parallel : bool
        Create plots in parallel
    exp_data : magine.data.ExperimentalData

    plot_type : str {'matplotlib', 'plotly'}

    """

    list_of_terms = list(data['term_name'].unique())
    fig_dict, to_remove = plot_genes_by_ont(data=data,
                                            list_of_terms=list_of_terms,
                                            save_name=save_name,
                                            out_dir=out_dir,
                                            exp_data=exp_data,
                                            run_parallel=run_parallel,
                                            plot_type=plot_type
                                            )

    for i in fig_dict:
        data.loc[data['term_name'] == i, 'term_name'] = fig_dict[i]

    data = data[~data['term_name'].isin(to_remove)]

    html_out = save_name + '_filter'
    ht.write_filter_table(data, html_out)


def plot_genes_by_ont(data, list_of_terms, save_name, out_dir=None,
                      exp_data=None, run_parallel=False, plot_type='plotly'):
    """ Creates a figure for each GO term in data

    BaseData should be a result of running calculate_enrichment.
    This function creates a plot of all proteins per term if a term is
    significant and the number of the reference set is larger than 5 and
    the total number of species measured is less than 100.


    Parameters
    ----------
    data : pandas.DataFrame
        previously ran enrichment analysis
    list_of_terms : list_list

    save_name : str
        name to save file
    out_dir : str
        output path for file
    exp_data : magine.ExperimentalData
        data to plot
    run_parallel : bool
        To run in parallel using pathos.multiprocessing
    plot_type : str
        plotly or matplotlib

    Returns
    -------
    out_array : dict
        dict where keys are pointers to figure locations
    """

    if out_dir is not None:
        if not os.path.exists(out_dir):
            os.mkdir(out_dir)
        if not os.path.exists(os.path.join(out_dir, 'Figures')):
            os.mkdir(os.path.join(out_dir, 'Figures'))

    data = data.copy()
    figure_locations = {}
    plots_to_create = []
    to_remove = set()

    if plot_type not in {'plotly', 'matplotlib'}:
        raise AssertionError("Please pass plotly or matplotlibn as plot_type")
    # filter data by significance and number of references
    if len(list_of_terms) == 0:
        print("No significant ontology terms!!!")
        return figure_locations, to_remove
    # here we are going to iterate through all sig GO terms and create
    # a list of plots to create. For the HTML side, we need to point to
    # a location
    _data = exp_data.data.copy()
    # create plot of genes over time
    for n, i in enumerate(list_of_terms):
        # want to plot all species over time
        index = data['term_name'] == i

        name = data[index]['term_name'].unique()

        if len(name) > 0:
            name = name[0]

        gene_set = set()
        genes = data[index]['genes']
        for g in genes:
            if isinstance(g, list):
                each = g
            else:
                each = g.split(',')

            gene_set.update(set(each))

        if plot_type == 'matplotlib':
            # too many genes isn't helpful on plots, so skip them
            if len(gene_set) > 100:
                figure_locations[i] = '<a>{0}</a>'.format(name)
                continue
        local_save_name = os.path.join('Figures',
                                       '{0}_{1}'.format(n, save_name))

        local_save_name = local_save_name.replace(':', '')
        out_point = '<a href="{0}.html">{1}</a>'.format(local_save_name, name)
        figure_locations[i] = out_point

        title = "{0} : {1}".format(str(i), name)
        local_df = _data.loc[_data[identifier].isin(list(gene_set))].copy()
        p_input = [local_df, list(gene_set), local_save_name, out_dir,
                   title, plot_type]

        plots_to_create.append(p_input)

    print("Starting to create plots for each term")
    _make_plots(plots_to_create, plot_species, run_parallel)

    return figure_locations, to_remove


def plot_dataframe(exp_data, html_filename, out_dir='proteins',
                   plot_type='plotly', run_parallel=False):
    """
    Creates

    Parameters
    ----------
    exp_data : magine.BaseData.
    html_filename : str
    out_dir: str, path
        Directory that will contain all proteins
    plot_type : str
        plotly or matplotlib output
    run_parallel : bool
        create plots in parallel
    Returns
    -------

    """
    if not os.path.exists(out_dir):
        os.mkdir(out_dir)
    local_data = exp_data.copy()
    species_to_plot = local_data[identifier].unique()

    fig_loc = {}
    plots = []

    suffix = 'html' if plot_type == 'plotly' else 'pdf'

    for i in species_to_plot:
        save_name = re.sub('[/_.]', '', i)

        plots.append([local_data, [i], save_name, out_dir, i, plot_type])

        n = '<a href="{0}/{1}.{2}">{1}</a>'.format(out_dir, save_name, suffix)
        fig_loc[i] = n

    _make_plots(plots, plot_species, run_parallel)

    # Place a link to the species for each key
    for key, value in fig_loc.items():
        local_data.loc[exp_data[identifier] == key, identifier] = value
    cols = [identifier, label_col, fold_change, p_val, sample_id, exp_method,
            flag]
    local_data = local_data[cols]
    ht.write_filter_table(local_data, html_filename)


def _make_plots(plots_to_make, plot_func, parallel=False):
    for i, _ in enumerate(plots_to_make):
        plots_to_make[i].append('pdf')
        plots_to_make[i].append(True)

    if parallel:
        st2 = time.time()
        pool = mp.Pool()
        # lambda a: function(a[0], **a[1]), arguments
        pool.map_async(lambda a: plot_func(*a), plots_to_make)
        pool.close()
        pool.join()
        end2 = time.time()
        print("parallel time = {}".format(end2 - st2))
        print("Done creating plots for each GO term")

    else:
        st1 = time.time()
        list(map(lambda a: plot_func(*a), plots_to_make))
        end1 = time.time()
        print("sequential time = {}".format(end1 - st1))
    plt.close('all')


def plot_species(df, species_list=None, save_name='test', out_dir=None,
                 title=None, plot_type='plotly', image_format='pdf',
                 close_plots=False):
    """

    Parameters
    ----------
    df: pandas.DataFrame
        magine formatted dataframe
    species_list: list
        List of genes to be plotter
    save_name: str
        Filename to be saved as
    out_dir: str
        Path for output to be saved
    title: str
        Title of plot, useful when list of genes corresponds to a GO term
    plot_type : str
        Use plotly to generate html output or matplotlib to generate pdf
    image_format : str
        pdf or png, only used if plot_type="matplotlib"
    close_plots : bool
        Close plot after making, use when creating lots of plots in parallel.
        
    Returns
    -------

    """

    ldf = df.copy()

    if out_dir is not None:
        if not os.path.exists(out_dir):
            os.mkdir(out_dir)

    # gather x axis points
    x_points = sorted(ldf[sample_id].unique())
    if len(x_points) == 0:
        return
    if isinstance(x_points[0], np.float):
        x_point_dict = {i: x_points[n] for n, i
                        in enumerate(x_points)}
    else:
        x_point_dict = {i: n for n, i
                        in enumerate(x_points)}
    if species_list is not None:
        ldf = ldf.loc[ldf[identifier].isin(species_list)].copy()

    ldf = log2_normalize_df(ldf, column=fold_change)

    n_plots = len(ldf[identifier].unique())
    num_colors = len(ldf[label_col].unique())
    color_list = sns.color_palette("tab20", num_colors)
    if plot_type == 'matplotlib':
        fig = plt.figure()
        ax = fig.add_subplot(111)
        ax.set_prop_cycle(plt.cycler('color', color_list))

    colors = enumerate(color_list)

    plotly = []
    names_list = []
    total_counter = 0
    for name, j in ldf.groupby(identifier):
        index_counter = 0
        for n, m in j.groupby(label_col):

            x = np.array(m[sample_id])
            if len(x) < 1:
                continue
            y = np.array(m['fold_change'])
            sig_flag = np.array(m[flag])
            index = np.argsort(x)
            x = x[index]
            y = y[index]
            s_flag = sig_flag[index]

            # x values with scaled values (only changes things if non-float
            # values are used for sample_id
            x_index = np.array([x_point_dict[ind] for ind in x])

            index_counter += 1
            total_counter += 1

            # create matplotlib plot
            if plot_type == 'matplotlib':
                label = "\n".join(wrap(n, 40))
                p = ax.plot(x_index, y, '.-', label=label)
                if len(s_flag) != 0:
                    color = p[0].get_color()
                    ax.plot(x_index[s_flag], y[s_flag], '^', color=color)

            # create plotly plot
            elif plot_type == 'plotly':
                c = next(colors)[1]
                plotly.append(_ploty_graph(x_index, y, n, n, c))
                if len(s_flag) != 0:
                    index_counter += 1
                    total_counter += 1
                    plotly.append(_ploty_graph(x_index[s_flag], y[s_flag],
                                               n, n, c, marker='x-open-dot'))
        names_list.append([name, index_counter])
    if plot_type == 'matplotlib':
        lgd = _format_mpl(ax, x_point_dict, x_points)
        if save_name is not None:
            tmp_savename = "{}.{}".format(save_name, image_format)
            if out_dir is not None:
                tmp_savename = os.path.join(out_dir, tmp_savename)
            plt.savefig(tmp_savename, bbox_extra_artists=(lgd,),
                        bbox_inches='tight')

        if close_plots:
            plt.close(fig)
        else:
            return fig

    elif plot_type == 'plotly':
        fig = _create_plotly(total_counter, n_plots, names_list, x_point_dict,
                             title, x_points, plotly)
        if save_name:
            _save_ploty_output(fig, out_dir, save_name)
        else:
            init_notebook_mode(connected=True)
            iplot(fig)


def _format_mpl(ax, x_point_dict, x_points):
    ax.set_xlim(min(x_point_dict.values()) - 2, max(x_point_dict.values()) + 2)
    ax.set_xticks(sorted(x_point_dict.values()))
    ax.set_xticklabels(x_points, rotation=90)
    plt.ylabel('log$_2$ Fold Change')

    plt.axhline(y=np.log2(1.5), linestyle='--')
    plt.axhline(y=-np.log2(1.5), linestyle='--')

    handles, labels = ax.get_legend_handles_labels()
    lgd = ax.legend(handles, labels, loc='best', ncol=3,
                    bbox_to_anchor=(1.01, 1.0))

    return lgd


def _create_plotly(total_counter, n_plots, names_list,
                   x_point_dict, title, x_points, plotly_list):
    true_list = [True] * total_counter
    scroll_list = [dict(args=['visible', true_list],
                        label='All',
                        method='restyle')]

    prev = 0
    # making all false except group defined by protein name
    for i in range(n_plots):
        t_row = [False] * total_counter
        for j in range(prev, prev + names_list[i][1]):
            t_row[j] = True
        prev += names_list[i][1]
        scroll = dict(args=['visible', t_row],
                      label=names_list[i][0], method='restyle')
        scroll_list.append(scroll)

    update_menu = list([dict(x=-0.05,
                             y=1,
                             yanchor='top',
                             buttons=scroll_list, )])
    ticks = np.sort(list(x_point_dict.values()))
    min_tick = np.min(ticks)
    max_tick = np.max(ticks)
    layout = plotly_graph.Layout(
        title=title,
        showlegend=True,
        xaxis=dict(title='Sample index',
                   range=[min_tick, max_tick],
                   showticklabels=True,
                   ticktext=x_points,
                   tickmode='array',
                   tickvals=ticks,
                   ),
        yaxis=dict(title='log2fc'),
        hovermode="closest",
        updatemenus=update_menu
    )

    return plotly_graph.Figure(data=plotly_list, layout=layout)


def _save_ploty_output(fig, out_dir, save_name):
    tmp_savename = "{}.html".format(save_name)
    if out_dir is not None:
        tmp_savename = os.path.join(out_dir, tmp_savename)

    x = plot(fig, filename=tmp_savename, auto_open=False,
             include_plotlyjs=False, output_type='div')
    ht.format_ploty(x, tmp_savename)


def _ploty_graph(x, y, label, enum, color, marker='circle'):
    """
    Creates a single scatter plot
    Parameters
    ----------
    x : list_like
    y : list_like
    label : str
    enum : int
    color : str
    marker : str

    Returns
    -------

    """
    l_color = 'rgba({},{},{},1.)'.format(color[0], color[1], color[2])
    if marker != 'circle':
        mode = 'markers'
        show = False
        size = 12
    else:
        mode = 'lines+markers'
        show = True
        size = 8
    legend = 'group_{}'.format(enum)

    g = plotly_graph.Scatter(
            x=x,
            y=y,
        hoveron='points',
            name=label,
            visible=True,
            mode=mode,
        legendgroup=legend,
            showlegend=show,
            line=dict(color=l_color),
            marker=dict(symbol=marker,
                        size=size,
                        color=l_color),
    )
    return g



import os

import networkx as nx
from nose.tools import ok_

from magine.networks.subgraphs import Subgraph
from magine.tests.sample_experimental_data import exp_data

_path = os.path.join(os.path.dirname(__file__), 'Network_files',
                     'sample_network.gml')


class TestSubgraphs(object):
    def __init__(self):
        self.network = nx.read_gml(_path)
        self.net_sub = Subgraph(self.network, exp_data)

    def test_downstream_nodes(self):
        """Test finding downstream nodes."""
        down_nodes = {'MTMR14', 'GABARAPL1', 'HMDB03850', 'GABARAPL2', 'EEA1',
                      'ATG16L1', 'ATG16L2', 'WIPI1', 'GABARAP', 'WIPI2',
                      'ATG12', 'ATG5', 'ZFYVE1'}
        for i in self.net_sub.downstream_of_node('MTMR14').nodes:
            ok_(i in down_nodes)

    def test_upstream_nodes(self):
        """Test finding upstream nodes."""
        up_nodes = {'MTMR4', 'MTMR14', 'MTMR3', 'ZFYVE1', 'HMDB03850'}
        for i in self.net_sub.upstream_of_node('ZFYVE1').nodes:
            ok_(i in up_nodes)

    def test_path_between_two_does_not_exist(self):
        start = 'HSPA9'
        end = 'ZFYVE1'
        g = self.net_sub.paths_between_pair(start, end, bidirectional=True)
        ok_(len(g.nodes) == 0)

    def test_path_between_two(self):
        start = 'AKT1'
        end = 'BAX'
        nodes = {'BCL2L1', 'PTK2', 'HMDB04249', 'BAX', 'MAPK14', 'AKT1',
                 'MAPK11', 'MAPK12', 'MAPK13', 'RAF1', 'MAP2K4',
                 'PIK3CA', 'PIK3CB', 'MAP3K1', 'PIK3CD', 'PIK3R3',
                 'PIK3R2', 'PIK3R1', 'TP53', 'CASP3', 'PTEN', 'PAK1',
                 'PAK2', 'BCL2'}
        g = self.net_sub.paths_between_pair(start, end,
                                            bidirectional=True)
        ok_(set(g.nodes()) == nodes)

    def test_paint_over_time(self):
        list_2 = {'CASP3', 'BAX', 'TP53'}
        g = self.net_sub.paths_between_list(list_2, draw=False,
                                            single_path=True,
                                            )
        colors = ['red'] * len(exp_data.sample_ids)
        # self.net_sub.measured_networks_over_time(g, colors, prefix='colored')

    def test_paint_over_time_up_down(self):
        list_2 = {'CASP3', 'BAX', 'TP53'}
        g = self.net_sub.paths_between_list(list_2, draw=False,
                                            single_path=True,
                                            )

        # self.net_sub.measured_networks_over_time_up_down(g, 'colored_updown')

    def test_paths_from_list(self):
        """Test finding paths from a list."""
        up_nodes = {'MTMR4', 'ZFYVE1', 'MTMR14', 'MTMR3', 'WIPI1'}
        edges = {('MTMR14', 'HMDB03850'),
                 ('HMDB03850', 'ZFYVE1'),
                 ('HMDB03850', 'WIPI1'),
                 ('MTMR4', 'HMDB03850'),
                 ('MTMR3', 'HMDB03850')}

        g = self.net_sub.paths_between_list(up_nodes, draw=False,
                                            save_name='ns_test')

        ok_(set(g.edges()) == edges)

        list_2 = {'CASP3', 'BAX', 'TP53'}
        g = self.net_sub.paths_between_list(list_2, draw=False,
                                            single_path=False,
                                            save_name='smaller_list')
        nodes = {'TP53', 'CASP3', 'CDKN1A', 'MAP3K1', 'BAX', 'MAPK10', 'MAPK8',
                 'MAPK9', 'BCL2'}
        ok_(set(g.nodes) == nodes)

    def test_neighbor(self):
        gene = 'BAX'
        g = self.net_sub.neighbors(gene, True, False, 1)
        layer_1_up = {'BAX', 'BCL2', 'BCL2L1', 'BCL2L11', 'BID', 'MAPK10',
                      'MAPK11', 'MAPK12', 'MAPK13', 'MAPK14', 'MAPK8',
                      'MAPK9', 'PRNP', 'SIRT1', 'TP53'}
        ok_(set(g.nodes) == layer_1_up)

        g = self.net_sub.neighbors(gene, True, False, 2)
        layer_2_up = {'BCL2L1', 'HSPA2', 'PMAIP1', 'HSPA6', 'MIR32', 'MIR30D',
                      'PPP2R3B', 'PPP2R3C', 'HSPA8', 'RAPGEF4', 'AKT1', 'AKT2',
                      'AKT3', 'ARAF', 'FCGR1A', 'RAPGEF3', 'MIR30A', 'MIR15A',
                      'EP300', 'HMDB03125', 'MAP3K1', 'PRNP', 'MAP3K7',
                      'PPP2R1B', 'MAP3K5', 'HSPA1A', 'MIR30C2', 'MIR30C1',
                      'RELA', 'KRAS', 'CTSH', 'DDIT3', 'CTSL', 'CTSO',
                      'PPP2R2D', 'CTSB', 'CTSC', 'CTSD', 'CTSF', 'PPP2R2B',
                      'MIR125B1', 'CTSZ', 'HMDB00902', 'MIR30B', 'MDM2',
                      'CTSS',
                      'MDM4', 'CTSV', 'CTSW', 'PPP2R3A', 'PPP2CA', 'PPP2CB',
                      'PPP2R5B', 'BMPR2', 'LAT', 'ATR', 'HRAS', 'EIF2AK4',
                      'TRAF3', 'PINK1', 'IL1R1', 'TRAF5', 'MIR223',
                      'MIR143', 'JUN', 'MAPK8IP1', 'CHEK2', 'CHEK1',
                      'SIRT1', 'NR4A1', 'FOS', 'PRKCD', 'RHOA', 'PTPN7',
                      'PTPN5', 'TP53', 'GZMB', 'TNF', 'PPP2R5D',
                      'PPP2R5E', 'CDC42', 'RALB', 'RALA', 'PPP2R5C',
                      'MAP2K1', 'PPP2R1A', 'MIR34A', 'ATM', 'RAP1B',
                      'RAP1A', 'PRKDC', 'MAPK14', 'TGFB3', 'MAPK10',
                      'MAPK11', 'MAPK12', 'MAPK13', 'RAF1', 'CREBBP',
                      'HMDB04947', 'MECOM', 'MAPK3', 'MAPK1', 'DUSP9',
                      'DUSP8', 'DUSP5', 'NFKB1', 'MAPK8', 'MAPK9',
                      'DUSP1', 'DUSP3', 'DUSP2', 'CREB1', 'PPP2R2C',
                      'DUSP4', 'USP7', 'NFKB2', 'MAPK8IP2', 'FOXO3',
                      'DUSP6', 'HSPA1B', 'EIF2AK1', 'EIF2AK3', 'EIF2AK2',
                      'BMPR1B', 'TP73', 'STAT5B', 'STAT5A', 'RIPK2',
                      'ERN1', 'MIR125A', 'MAP2K6', 'TGFB1', 'TGFB2',
                      'PTK2', 'ACVR1', 'MIR30E', 'FCGR3A', 'BAX', 'MCL1',
                      'FCGR3B', 'MAP2K3', 'BMPR1A', 'BAD', 'CTSK', 'MAP2K7',
                      'MAP2K4', 'BBC3', 'HSPA1L', 'RCHY1', 'CREB3L2',
                      'BCL2L11', 'CREB3L3', 'BID', 'SOD1', 'NRAS',
                      'CREB3L1', 'CREB5', 'TRAF1', 'TRAF2', 'NOS3',
                      'RAC2', 'RAC3', 'TRAF6', 'RAC1', 'PRKCA', 'CREB3',
                      'PRKCB', 'PRKCE', 'FCGR2C', 'FCGR2A', 'PRKCZ',
                      'HMDB03747', 'IL1RAP', 'DUSP16', 'DUSP10', 'PTPRR',
                      'MIR125B2', 'CASP8', 'PPP2R5A', 'PPP2R2A', 'MAP3K11',
                      'DUSP7', 'BRAF', 'CREB3L4', 'ATF4', 'ATF6B', 'ATF2',
                      'BCL2'}

        new_nodes = set(g.nodes)

        ok_(set(g.nodes) == layer_2_up)

        g = self.net_sub.neighbors(gene, False, True, 1)
        layer_1_down = {'CASP3', 'CYCS', 'BAX', 'CAPN2', 'CAPN1', 'BCL2'}
        ok_(set(g.nodes) == layer_1_down)

        g = self.net_sub.neighbors(gene, False, True, 2)
        layer_2_down = {'ACTG1', 'SPTAN1', 'CYCS', 'SPTA1', 'CASP12',
                        'CASP9', 'NLRP1', 'BIK', 'MAP3K1', 'TLN2', 'DCC',
                        'TLN1', 'CDK5R1', 'CAPN2', 'CAPN1', 'APAF1',
                        'PARP4', 'DFFA', 'PARP2', 'PARP1', 'STK3', 'STK4',
                        'BAX', 'PARP3', 'CASP7', 'TP53', 'CASP3', 'BAD',
                        'BAK1', 'ACTB', 'PAK1', 'PAK2', 'BCL2'}

        ok_(set(g.nodes) == layer_2_down)

    def test_expand(self):
        g = nx.DiGraph()
        g.add_edge('BCL2L1', 'BAX')
        g.add_edge('MAPK14', 'BAX')

        ng = self.net_sub.expand_neighbors(network=g, nodes=['BAX'],
                                           downstream=True)

        trues = {
            'BCL2L1', 'CASP3', 'CAPN2', 'BAX', 'MAPK14', 'CYCS',
            'CAPN1', 'BCL2'
        }

        ok_(trues == set(ng.nodes))

        includes = ['CASP3', 'CAPN2']

        ng = self.net_sub.expand_neighbors(g, 'BAX', downstream=True,
                                           include_only=includes)

        trues = {
            'BCL2L1', 'CASP3', 'CAPN2', 'BAX', 'MAPK14',
        }
        ok_(trues == set(ng.nodes))


if __name__ == '__main__':
    t = TestSubgraphs()
    t.test_neighbor()



import json
import sys
import tempfile

import networkx as nx
import pydot


def nx_to_igraph(network):
    try:
        import igraph
    except ImportError:
        raise ImportError('requires igraph ')
    file_descriptor, file_path = tempfile.mkstemp(suffix='.gml')

    nx.write_gml(network, file_path)
    igraph_network = igraph.read(file_path)
    return igraph_network


def nx_to_json(network):
    """
    Convert from networkx Digraph to cytoscape json format
    Parameters
    ----------
    network : nx.DiGraph

    Returns
    -------
    dict
    """
    nodes, edges = [], []

    for node_id in network.nodes:
        node = network.node[node_id]
        new_node = {}
        node_columns = node.keys()
        data = {col: node[col] for col in node_columns if col != 0}
        if 'position' in node.keys():
            new_node['position'] = node['position']
        data['id'] = str(node_id)
        data['name'] = data['id']
        new_node['data'] = data
        nodes.append(new_node)

    for source, target, data in network.edges(data=True):
        data['source'] = str(source)
        data['target'] = str(target)
        edges.append({'data': data})

    return {'nodes': json.dumps(nodes),
            'edges': json.dumps(edges)}


def nx_to_dot(graph):
    graph = format_to_directions(graph)
    # set Graphviz graph type
    if graph.is_directed():
        graph_type = 'digraph'
    else:
        graph_type = 'graph'
    strict = graph.number_of_selfloops() == 0 and not graph.is_multigraph()

    name = graph.name
    if name is not '':
        name = make_str(name)
    args = {make_str(i): make_str(graph.graph[i]) for i in graph.graph}
    g = pydot.Dot(name, graph_type=graph_type, strict=strict, **args)
    try:
        g.set_node_defaults(**{make_str(i): make_str(graph.graph[i]) for i in
                               graph.graph['node']})
    except KeyError:
        pass
    try:
        g.set_edge_defaults(**{make_str(i): make_str(graph.graph[i]) for i in
                               graph.graph['edge']})
    except KeyError:
        pass

    for n, data in graph.nodes(data=True):
        str_data = dict((make_str(k), make_str(v)) for k, v in data.items())
        g.add_node(pydot.Node(make_str(n), **str_data))

    for u, v, data in graph.edges(data=True):
        str_data = dict((make_str(k), make_str(v)) for k, v in data.items()
                        if k != 'key')
        g.add_edge(pydot.Edge(make_str(u), make_str(v), **str_data))
    return g


def export_to_dot(graph, save_name, image_format='png', engine='dot'):
    """
    Converts networkx graph to graphviz dot

    Parameters
    ----------
    graph : networkx.DiGraph
    save_name : str
        name of file to save
    image_format : str
        format of output( pdf, png, svg)
    engine : str
        graphviz engine
            dot, twopi,

    Returns
    -------

    """

    py_dot = nx_to_dot(graph)
    if save_name.endswith('.png'):
        basename = save_name[:-4]
    else:
        basename = save_name
    py_dot.write('{}.{}'.format(basename, image_format), prog=engine,
                 format=image_format)


def check_graphviz(network):
    if isinstance(network, nx.DiGraph):
        network = format_to_directions(network)

    return nx_to_dot(network)


def format_to_directions(network):
    activators = ['activate', 'expression', 'phosphorylate']
    inhibitors = [
        'inhibit', 'repression', 'dephosphorylate', 'deubiquitinate',
        'ubiquitinate'
    ]
    physical_contact = ['binding', 'dissociation', 'stateChange',
                        'compound', 'glycosylation']
    indirect_types = ['indirect']
    for source, target, data in network.edges(data=True):
        if 'interactionType' in data:
            edge_type = data['interactionType']
            for j in activators:
                if j in edge_type:
                    network[source][target]['arrowhead'] = 'normal'
            for j in inhibitors:
                if j in edge_type:
                    network[source][target]['arrowhead'] = 'tee'

            for j in physical_contact:
                if j in edge_type:
                    network[source][target]['dir'] = 'both'
                    network[source][target]['arrowtail'] = 'diamond'
                    network[source][target]['arrowhead'] = 'diamond'

            for j in indirect_types:
                if j in edge_type:
                    network[source][target]['arrowhead'] = 'diamond'
                    network[source][target]['style'] = 'dashed'
    return network


if sys.version_info[0] == 2:
    def make_str(x):
        """Return the string representation of t."""
        if isinstance(x, unicode):
            return x
        else:
            return unicode(str(x), 'unicode-escape')
else:
    def make_str(x):
        """Return the string representation of t."""
        return str(x)



import os

import seaborn as sns

from magine.networks.exporters import nx_to_igraph
from magine.networks.utils import add_attribute_to_network

try:
    basestring
# Allows isinstance(foo, basestring) to work in Python 3
except:
    basestring = str


def draw_igraph(mol_net, save_name=None, layout='auto', title=None,
                positions=None, cluster=False, node_size=50,
                bbox=None, margin=None, inline=False, node_font_size=24,
                font_size=36):
    """

    Parameters
    ----------

    mol_net : networkx.DiGraph
        networkx graph
    save_name : str
        Save name
    layout : str
        Options = "kk", "fr", "drl", "lgl", "tree", "graphopt", "mds", "sugiyama",
        "auto", "grid_fr"
    title : str, optional
    positions : list, optional
    bbox : list
    cluster : bool
        Add a shape around species that are tagged the same.
         g.node[0][term_name]='dna'
         g.node[0][term_name]='dna'
         This will be grouped together.
    margin : list
        Margin of white space for boxes. This should be considered for the
         labels of the nodes
    node_size : int
    node_font_size: int
        Size of node labels
    font_size : int
        Title and group name (if clustering) font size
    inline : bool
        Return plot as svg view ipython

    Returns
    -------

    """
    try:
        import cairo
    except ImportError:
        ImportError("Please install pycairo to use igraph plotting")

    try:
        import igraph
        from igraph.drawing.text import TextDrawer
        from igraph.drawing.colors import color_name_to_rgba
    except ImportError:
        raise ImportError("No igraph, cannot use plotting function")

    if isinstance(mol_net, igraph.Graph):
        g = mol_net
    else:
        g = nx_to_igraph(mol_net)
        if not isinstance(g, igraph.Graph):
            print("Error converting to Igraph")
            return
    if inline:
        if bbox is None:
            bbox = [500, 500]
        if margin is None:
            margin = [50, 50, 50, 50]

    if bbox is None:
        bbox = [2400, 2400]
    if margin is None:
        margin = [50, 50, 50, 50]

    _valid_layouts = {
        "kk", "fr", "drl", "lgl", "tree", "graphopt", "mds", "sugiyama",
        "auto", "grid_fr",
    }
    if layout not in _valid_layouts:
        raise Exception('layout {} not in {}'.format(layout, _valid_layouts))

    mark_groups = None
    membership = None

    if cluster and 'termName' in g.vs.attributes():
        cl = igraph.VertexClustering(g).FromAttribute(g, attribute='termName')
        membership = cl.membership
        if membership is not None:
            gcopy = g.copy()
            # edges = []
            # for edge in g.es():
            #     if membership[edge.tuple[0]] != membership[edge.tuple[1]]:
            #         edges.append(edge)
            # gcopy.delete_edges(edges)
            if positions is None:
                positions = gcopy.layout(layout)
            n_clusters = len(set(membership))
            rgb_colors = sns.color_palette("tab20", n_clusters)
            colors = rgb_colors.as_hex()
            mark_groups = dict()
            for n, color in enumerate(colors):
                mem = tuple(i for i, j in enumerate(membership) if j == n)
                mark_groups[mem] = color

    if positions is None:
        positions = g.layout(layout)

    visual_style = dict()
    visual_style["vertex_label_dist"] = 0
    visual_style["vertex_shape"] = "circle"
    visual_style["vertex_size"] = node_size
    visual_style["vertex_label_size"] = node_font_size
    visual_style["layout"] = positions
    visual_style["margin"] = margin
    # visual_style["edge_curved"] = True

    if 'color' in g.es.attributes():
        visual_style["edge_color"] = g.es["color"]

    if save_name is not None:
        if not save_name.endswith('.png'):
            save_name += '.png'

    if cluster:
        # add some white space to add labels
        bbox_plus_margin = [bbox[0] + bbox[0] * .45, bbox[1] + bbox[0] * .1]
        margin[2] += bbox[0] * .25
        margin[0] += bbox[0] * .05
    else:
        bbox_plus_margin = bbox

    # create entire surface
    plot = igraph.Plot(save_name,
                       bbox=bbox_plus_margin,
                       background='white')

    # add plot to surface
    plot.add(g, mark_groups=mark_groups, bbox=bbox, **visual_style)

    # have to redraw to add the plot
    plot.redraw()

    # Grab the surface, construct a drawing context and a TextDrawer
    ctx = cairo.Context(plot.surface)
    ctx.set_font_size(font_size)

    if title is not None:
        drawer = TextDrawer(ctx, title, halign=TextDrawer.CENTER)
        drawer.draw_at(0, 40, width=600)

    if membership is not None:
        labels = dict()
        for vertex in g.vs():
            name = vertex['termName'].replace(',', '\n')
            if name in labels:
                continue
            labels[name] = rgb_colors[membership[vertex.index]]
        spacing = bbox[1] / len(labels)
        ctx.set_font_size(24)
        for n, (label, rgb_c) in enumerate(labels.items()):
            text_drawer = TextDrawer(ctx, text=label, halign=TextDrawer.LEFT)
            x_coord = bbox[0] + 25
            y_coord = 100 + (n * spacing)
            text_drawer.draw_at(x=x_coord, y=y_coord, width=300, wrap=True)
            ctx.set_source_rgba(*rgb_c)
            ctx.arc(x_coord - 1.5 * node_size, y_coord, node_size, 0, 6.28)
            ctx.fill()
            ctx.set_source_rgba(0, 0, 0, 1)
    # Save the plot
    if save_name is not None:
        plot.save()
    if inline:
        return plot

    return plot, positions


def paint_network_overtime(graph, exp_data, color_list, save_name,
                           compile_images=False, cluster=False, fig_width=1500,
                           fig_height=1500, layout='auto', inline=False):
    """
    Adds color attribute to network over time.

    Parameters
    ----------
    compile_images
    graph : nx.DiGraph
        Network
    exp_data : list_like
        List of lists, where the inner list contains the node to add the
        color
    color_list : list_like
        list of colors for each time point
    save_name : str
        prefix for images to be saved
    cluster : bool
        Group nodes based on category 'term_name'
    fig_width : int
        Size of fig
    fig_height : int
        Size of fig
    layout : str
        Layout format for igraph
    Returns
    -------

    """

    labels = []
    measured_list = []
    for i, j in zip(exp_data.species.sig.sample_ids,
                    exp_data.species.sig.by_sample):
        measured_list.append(j)
        labels.append(i)
    if isinstance(color_list, basestring):
        color_list = [color_list, ] * len(measured_list)
    if len(measured_list) != len(color_list):
        print('Length of list'
              'of data must equal len of color list')
        return

    string = 'convert -delay 100 '
    tmp_graph = graph.copy()
    pos = None
    for n, i in enumerate(measured_list):
        graph2 = add_attribute_to_network(tmp_graph, i, 'color', color_list[n],
                                          'white')

        s_name = '%s_%04i_igraph.png' % (save_name, n)
        result, pos = draw_igraph(graph2, s_name, positions=pos, node_size=50,
                                  bbox=[fig_width, fig_height],
                                  margin=[100, 100, 100, 100],
                                  cluster=cluster, title=labels[n],
                                  layout=layout)
        if inline:
            from IPython.display import Image, display
            display(Image(s_name))
        string += ' ' + s_name
        # yield result
    string1 = string + '  %s.gif' % save_name
    string2 = string + '  %s.pdf' % save_name

    if compile_images:
        os.system(string1)
        os.system(string2)


def color_by_list(graph, species_list, labels, colors, save_name,
                  compile_images=False, cluster=False, fig_width=1500,
                  fig_height=1500, layout='auto', inline=False, **kwargs):
    """
    Adds color attribute to network over time.

    Parameters
    ----------
    compile_images
    graph : nx.DiGraph
        Network
    species_list : list_like
        List of lists, where the inner list contains the node to add the
        color
    labels : list_like
        List of labels
    colors : list_like
        list of colors for each time point
    save_name : str
        prefix for images to be saved
    cluster : bool
        Group nodes based on category 'term_name'
    fig_width : int
        Size of fig
    fig_height : int
        Size of fig
    layout : str
        Layout format for igraph
    inline : bool
        Display figures inline (for ipython/jupyter)
    Returns
    -------

    """
    if inline:
        from IPython.display import Image, display
    if isinstance(colors, str):
        colors = [colors, ] * len(species_list)
    if len(species_list) != len(colors):
        print('Length of list'
              'of data must equal len of color list')
        return
    if labels is None:
        labels = [None] * len(species_list)
    string = 'convert -delay 100 '
    tmp_graph = graph.copy()
    pos = None
    for n, i in enumerate(species_list):
        graph2 = add_attribute_to_network(tmp_graph, i, 'color', colors[n],
                                          'white')

        s_name = '%s_%04i_igraph.png' % (save_name, n)
        result, pos = draw_igraph(graph2, s_name, positions=pos,
                                  title=labels[n], **kwargs)
        if inline:
            display(Image(s_name))
        string += ' ' + s_name
        # yield result
    string1 = string + '  %s.gif' % save_name
    string2 = string + '  %s.pdf' % save_name

    if compile_images:
        os.system(string1)
        os.system(string2)


if __name__ == '__main__':
    import networkx as nx
    import igraph

    # g = nx.read_gpickle('../background_network.p.gz')
    g = nx.DiGraph()
    g.add_edge("a", "b")
    print(type(g))
    draw_igraph(g, save_name='test', layout='lgl')



try:
    basestring
# Allows isinstance(foo, basestring) to work in Python 3
except:
    basestring = str

from bioservices import HGNC, UniProt
import pandas as pd
from sortedcontainers import SortedSet, SortedDict

from magine.mappings.databases import load_hgnc, load_uniprot, load_ncbi

pd.set_option('display.width', 20000)


class GeneMapper(object):
    """
    Mapping class between common gene ids

    Database was creating by pulling down from NCBI, UNIPROT, HGNC

    """

    def __init__(self, species='hsa'):
        self.species = species
        self.hgnc = load_hgnc()
        self.ncbi = load_ncbi()
        self.uniprot = load_uniprot()
        self._gene_name_to_uniprot = None
        self._gene_name_to_alias_name = None
        self._gene_name_to_ensembl = None
        self._gene_name_to_kegg = None
        self._uniprot_to_gene_name = None
        self._uniprot_to_kegg = None
        self._kegg_to_gene_name = None
        self._kegg_to_uniprot = None
        self._ncbi_to_symbol = None
        # hackish way to initilize
        if 'x' in self.kegg_to_gene_name:
            pass
        if 'x' in self.ncbi_to_symbol:
            pass
        if 'x' in self.gene_name_to_uniprot:
            pass

    @property
    def gene_name_to_uniprot(self):
        if self._gene_name_to_uniprot is None:
            self._gene_name_to_uniprot = _dict(self.hgnc, 'symbol',
                                               'uniprot_ids')
        return self._gene_name_to_uniprot

    @property
    def gene_name_to_alias_name(self):
        if self._gene_name_to_alias_name is None:
            self._gene_name_to_alias_name = _dict(self.hgnc, 'symbol',
                                                  'alias_name')
        return self._gene_name_to_alias_name

    @property
    def gene_name_to_ensembl(self):
        if self._gene_name_to_ensembl is None:
            self._gene_name_to_ensembl = _dict(self.hgnc, 'symbol',
                                               'ensembl_gene_id')
        return self._gene_name_to_ensembl

    @property
    def uniprot_to_gene_name(self):
        if self._uniprot_to_gene_name is None:
            self._uniprot_to_gene_name = _dict(self.hgnc, 'uniprot_ids',
                                               'symbol')
        return self._uniprot_to_gene_name

    @property
    def gene_name_to_kegg(self):
        if self._gene_name_to_kegg is None:
            self._gene_name_to_kegg = _dict(self.uniprot, 'Gene_Name', 'KEGG')
        return self._gene_name_to_kegg

    @property
    def uniprot_to_kegg(self):
        if self._uniprot_to_kegg is None:
            self._uniprot_to_kegg = _dict(self.uniprot, 'uniprot', 'KEGG')
        return self._uniprot_to_kegg

    @property
    def kegg_to_gene_name(self):
        if self._kegg_to_gene_name is None:
            self._kegg_to_gene_name = _dict(self.uniprot, 'KEGG', 'Gene_Name')
            self._kegg_to_gene_name.update(manual_dict)
        return self._kegg_to_gene_name

    @property
    def kegg_to_uniprot(self):
        if self._kegg_to_uniprot is None:
            self._kegg_to_uniprot = _dict(self.uniprot, 'KEGG', 'uniprot')
        return self._kegg_to_uniprot

    @property
    def ncbi_to_symbol(self):
        if self._ncbi_to_symbol is None:
            self._ncbi_to_symbol = _dict(self.ncbi, 'GeneID', 'Symbol')
        return self._ncbi_to_symbol

    def check_synonym_dict(self, term, format_name):
        """ checks hmdb database for synonyms and returns formatted name

        Parameters
        ----------
        term : str
        format_name : str

        Returns
        -------
        dict

        """
        index = 'alias_symbol'
        synonyms = self.hgnc.copy()
        synonyms = synonyms.loc[~synonyms[index].isna()]
        synonyms[index] = synonyms[index].str.upper()

        hits = synonyms.loc[synonyms[index].str.contains(term.upper())].copy()
        hits[index] = hits[index].str.split('|')

        for i, row in hits.iterrows():
            if term in row[index]:
                return [row[format_name]]
        matches = sorted(set(hits[format_name].values))
        return matches

    def convert_kegg_nodes(self, network, species='hsa'):
        """ Convert kegg ids to HGNC gene symbol.

        Parameters
        ----------
        network : nx.DiGraph
        species : str {'hsa'}
            Main support for humans only.

        Returns
        -------
        kegg_to_gene_name, kegg_short : dict, dict
        """

        # Create the dictionary to store all conversions to be returned
        missing = set()
        unknown_genes = set()
        hits = {i for i in set(network.nodes) if i.startswith(species)}
        prefix = species + ':'
        kegg_short = {i: i.lstrip(prefix) for i in hits}

        kegg_to_gene_name = {i: self.kegg_to_gene_name[i]
                             for i in hits if i in self.kegg_to_gene_name}
        for gene, gn in kegg_to_gene_name.items():
            if isinstance(gn, basestring):
                kegg_to_gene_name[gene] = gn
            elif len(gn) == 1:
                kegg_to_gene_name[gene] = gn[0]
            else:
                found = False
                for g in gn:
                    if g in self.gene_name_to_uniprot:
                        kegg_to_gene_name[gene] = g
                        found = True
                if not found:
                    print("Found species that cannot be mapped to HGNC.")
                    print(gene, gn)
                    # kegg_to_gene_name[gene] = gn[0]
                    missing.add(gene)
        missing.update(hits.difference(kegg_to_gene_name))

        # check stores dictionaries
        for gene in missing:
            name_stripped = gene.lstrip(prefix)
            if int(name_stripped) in self.ncbi_to_symbol:
                new = self.ncbi_to_symbol[int(name_stripped)][0]
                if not isinstance(new, float):
                    kegg_to_gene_name[gene] = new
                else:
                    unknown_genes.add(gene)
            else:
                unknown_genes.add(gene)

        if len(unknown_genes) > 0:
            add_dict, missed = self.kegg_to_symbol_through_uniprot(
                unknown_genes)
            kegg_to_gene_name.update(add_dict)

            add_dict, final_missed = self.kegg_to_hugo(missed, species)
            kegg_to_gene_name.update(add_dict)

            print("{} mappings not found from kegg to"
                  " gene name".format(len(final_missed)))
            print(final_missed)

        return kegg_to_gene_name, kegg_short

    def kegg_to_symbol_through_uniprot(self, unknown_genes):
        # create string to call uniprot for mapping
        search_string = '\t'.join(unknown_genes)
        kegg_to_gene_name = dict()
        missing = set()
        uniprot = UniProt(verbose=True)
        # This is where it gets tricky. Checking to see if there is a uniprot
        # mapping for the species, if not, trying from KEGG side. Sometimes
        # kegg  links to a different uniprot, or uniprot links to a diff kegg.
        uni_dict = dict(uniprot.mapping("KEGG_ID", "ACC", query=search_string))
        for i in unknown_genes:
            if i in uni_dict:
                for n in uni_dict[i]:
                    x = uniprot.search("accession:{}".format(n),
                                       columns='genes(PREFERRED),reviewed,id',
                                       limit=1)
                    _, data = x.rstrip('\n').split('\n')
                    name, review, entry = data.split('\t')
                    if n != entry:
                        print(i, n, entry, x, "dont match")
                    elif review == 'reviewed':
                        kegg_to_gene_name[i] = name

            else:
                missing.add(i)
        print("{} mappings not found from kegg to"
              " gene name".format(len(missing)))
        print(missing)
        return kegg_to_gene_name, missing

    def kegg_to_hugo(self, genes, species='hsa'):
        """
        Converts all KEGG names to HGNC

        Parameters
        ----------
        genes : list
        species : str

        Returns
        -------
        dict
        """
        prefix = species + ':'
        hugo = HGNC(verbose=True)
        hugo_dict = {}
        not_found = set()
        for i in genes:
            tmp_name = i.lstrip(prefix)
            mapping = hugo.search(tmp_name)
            if 'response' in mapping:
                response = mapping['response']
                if 'numFound' in response:
                    if response['numFound'] == 0:
                        not_found.add(i)
                        continue
                    elif response['numFound'] == 1:
                        docs = response['docs'][0]
                        hugo_dict[i] = docs['symbol']
                        continue
                    else:
                        if 'symbol' in response['docs'][0]:
                            hugo_dict[i] = response['docs'][0]['symbol']
            else:
                not_found.add(i)
        if not_found != 0:
            print("{} not found after HGNC mapping".format(len(not_found)))
            print("{} ".format(not_found))
        return hugo_dict, not_found


def _dict(data, key, value):
    """
    creates a dictionary with a list of values for each key

    Parameters
    ----------
    data : pandas.DataFrame
    key : str
    value : str

    Returns
    -------

    """
    return_dict = SortedDict()

    d = data[[key, value]].copy()
    d.dropna(how='any', inplace=True)

    for i, j in d.values:
        if i in return_dict:
            return_dict[i].add(j)
        else:
            return_dict[i] = SortedSet([j])
    return return_dict


manual_dict = {'hsa:857': 'CAV1',
               'hsa:2250': 'FGF5',
               'hsa:5337': 'PLD1',
               'hsa:4312': 'MMP1',
               'hsa:102723407': 'IGHV4OR15-8',
               'hsa:100132074': 'FOXO6',
               'hsa:728635': 'DHRS4L1',
               'hsa:10411': 'RAPGEF3',
               'hsa:100101267': 'POM121C',
               'hsa:2768': 'GNA12',
               'hsa:2044': 'EPHA5',
               'hsa:100533467': 'BIVM-ERCC5',
               'hsa:7403': 'KDM6A',
               'hsa:1981': 'EIF4G1',
               'hsa:2906': 'GRIN2D',
               'hsa:4088': 'SMAD3',
               'hsa:6776': 'STAT5A',
               'hsa:182': 'JAG1',
               'hsa:3708': 'ITPR1',
               'hsa:1293': 'COL6A3',
               'hsa:93034': 'NT5C1B',
               'hsa:574537': 'UGT2A2',
               'hsa:11044': 'PAPD7',
               'hsa:57292': 'KIR2DL5A',
               'hsa:107984026': 'ACAP3'
               }

if __name__ == '__main__':
    gm = GeneMapper()



import pandas as pd
from nose.tools import raises, ok_

from magine.data.base import BaseData


class ConcentrationBaseData(BaseData):
    def __init__(self, *args, **kwargs):
        super(ConcentrationBaseData, self).__init__(*args, **kwargs)
        self._index = 'protein'

    @property
    def _constructor(self):
        return ConcentrationBaseData


def load_concentration(file):
    dataframe = pd.read_csv(file)
    return ConcentrationBaseData(dataframe)


def test_pivot():
    index = 'protein'
    values = 'treated_control_fold_change'
    columns = 'time_points'
    x = [
        {values: 1, index: 'x', columns: '1'},
        {values: -2, index: 'i', columns: '2'},
        {values: -4, index: 'b', columns: '1'},
        {values: -4, index: 'b', columns: '2'},
    ]

    d = ConcentrationBaseData(x)
    df = d.pivoter(True, values=values, columns=columns)

    ok_(df.shape == (3, 2))


@raises(AssertionError)
def test_min_raises():
    index = 'protein'
    values = 'treated_control_fold_change'
    columns = 'time_points'
    x = [
        {values: 1, index: 'x', columns: '1'},
        {values: -2, index: 'i', columns: '2'},
        {values: -4, index: 'b', columns: '1'},
        {values: -4, index: 'b', columns: '2'},
    ]

    d = ConcentrationBaseData(x)
    d.require_n_sig(index=index, columns=columns)


def test_filter_min():
    index = 'protein'
    values = 'treated_control_fold_change'
    columns = 'time_points'
    flag = 'significant'
    x = [
        {values: 1, index: 'x', columns: '1', flag: True, 'x': 2},
        {values: -2, index: 'i', columns: '2', flag: True, 'x': 2},
        {values: -4, index: 'b', columns: '1', flag: True, 'x': 1},
        {values: -4, index: 'b', columns: '2', flag: True, 'x': 1},
    ]

    d = ConcentrationBaseData(x)
    df = d.require_n_sig(columns=columns, n_sig=1)
    ok_(df.shape[0] == 4)
    df = d.require_n_sig(columns=columns, n_sig=2)
    ok_(df.shape[0] == 2)
    df = d.require_n_sig(columns=columns, n_sig=3)
    ok_(df.shape[0] == 0)

    x = [
        {values: 1, index: 'x', columns: '1', flag: True, 'x': 2},
        {values: -4, index: 'b', columns: '1', flag: True, 'x': 1},
        {values: -2, index: 'i', columns: '2', flag: True, 'x': 2},
        {values: -4, index: 'b', columns: '2', flag: True, 'x': 1},
        {values: -4, index: 'b', columns: '2', flag: False, 'x': 2},
    ]

    d = ConcentrationBaseData(x)

    df = d.require_n_sig(index=index, columns=columns,
                         n_sig=2)

    ok_(df.shape == (3, 5))


def test_log_normal():
    x = [['a', 2],
         ['b', -2],
         ['c', -16],
         ['d', 16]]

    df = ConcentrationBaseData(x, columns=['name', 'value'])
    new_df = df.log2_normalize_df('value')

    ok_(new_df[new_df['name'] == 'a']['value'].values == [1.])
    ok_(new_df[new_df['name'] == 'c']['value'].values == [-4.])



import os
import tempfile

import magine.html_templates.html_tools as html_tools
from magine.enrichment.enrichment_result import load_enrichment_csv


def test_filter():
    df = load_enrichment_csv(os.path.join(
        os.path.dirname(__file__), 'Data', 'enrichr_test_enrichr.csv')
    )
    html_tools.write_filter_table(df, 'tmp')
    html_tools.write_filter_table(df, tempfile.mkdtemp())



layouts = {
    'breadthfirst': {
        'name': 'breadthfirst',
        'directed': 'true',
        'spacingFactor': 1.5
    },
    'cose': {
        'name': 'cose',
        'spacingFactor': 1.5
    },
    'cose-bilkent': {
        'name': 'cose-bilkent',
        'spacingFactor': 1.5,
        'animate': False
    },
    'dagre': {
        'name': 'dagre',
        'rankDir': 'LR',
        'spacingFactor': 1.5
    },
    'concentric': {
        'name': 'concentric',
        'spacingFactor': 1.5

    }
}

styles = {
    'default':
        [
            {"selector": "node",
             "css": {"text-opacity": 1.0,
                     "background-opacity": 1.0,
                     "font-weight": "normal",
                     "color": "rgb(0,153,234)",
                     "border-width": 3.0,
                     "border-color": "rgb(51,51,51)",
                     "font-size": 9,
                     "height": 50,
                     "width": 50,
                     "label": "data(name)",

                     "text-wrap": "wrap",
                     "text-max-width": 50,
                     "shape": "ellipse",
                     "background-color": "data(color)",
                     "text-halign": "center",
                     "font-family": "SansSerif",
                     "text-valign": "center",
                     "border-opacity": 1.0
                     },
             },
            {'selector': "node[speciesType = 'compound']",
             'css': {"text-opacity": 1.0,
                     "background-opacity": 1.0,
                     "font-weight": "normal",
                     "color": "rgb(0,153,234)",
                     "border-width": 3.0,
                     "border-color": "rgb(51,51,51)",
                     "font-size": 9,
                     "height": 35,
                     "width": 35,
                     'label': 'data(chemName)',

                     "text-wrap": "wrap",
                     "text-max-width": 95,
                     "shape": "ellipse",
                     "background-color": "rgb(255,255,255)",
                     "text-halign": "center",
                     "font-family": "SansSerif",
                     "text-valign": "center",
                     "border-opacity": 1.0},
             },

            {"selector": "$node > node",
             "css": {"font-size": 20,
                     "shape": "ellipse",
                     "padding-right": "10px",
                     "padding-bottom": "10px",
                     "padding-top": "10px",
                     "text-valign": "top",
                     "text-halign": "center",
                     "background-color": "#bbb",
                     "padding-left": "10px"},
             },
            {"selector": "node:selected",
             "css": {"background-color": "rgb(255,0,102)"},
             },

            {"selector": "edge",
             "css": {"opacity": 1.0,
                     "text-opacity": 1.0,
                     "font-size": 12,
                     "font-weight": "normal",

                     'width': '2',

                     "color": "rgb(0,0,0)",
                     "line-color": "rgb(51,51,51)",
                     "source-arrow-color": "rgb(0,0,0)",
                     "target-arrow-color": "rgb(51,51,51)",

                     "curve-style": "bezier",
                     "line-style": "solid",
                     "target-arrow-shape": "triangle",
                     "source-arrow-shape": "none",

                     "font-family": "SansSerif"

                     },
             },

            {"selector": "edge[weight>1]",
             "css": {"opacity": 1.0,
                     "text-opacity": 1.0,
                     "font-size": 12,
                     "font-weight": "normal",

                     # 'width': 'mapData(width, 1, 10, 1, 10)',
                     'width': 'data(width)',

                     "color": "rgb(0,0,0)",
                     "line-color": "rgb(51,51,51)",
                     "source-arrow-color": "rgb(0,0,0)",
                     "target-arrow-color": "rgb(51,51,51)",

                     "curve-style": "bezier",
                     "line-style": "solid",
                     "target-arrow-shape": "triangle",
                     "source-arrow-shape": "none",

                     "font-family": "SansSerif"

                     },
             },
            {"selector": "edge[interactionType *= 'inhibit']"
                         ",edge[interactionType *= 'deactivat']",
             "css": {
                 "text-opacity": 1.0,
                 "font-size": 12,
                 "font-weight": "normal",
                 "font-family": "SansSerif",

                 'curve-style': 'bezier',
                 'source-arrow-shape': 'none',
                 'target-arrow-shape': 'tee',

                 "color": "rgb(0,0,0)",
                 "line-color": "rgb(51,51,51)",
                 "source-arrow-color": "rgb(0,0,0)",
                 "target-arrow-color": "rgb(51,51,51)",
                 "line-style": "solid",

             },
             },
            {"selector": "edge:selected",
             "css": {
                 "line-color": "rgb(255,0,0)",
                 "label": "data(interactionType)",
             },
             },
            {'selector': 'edge[weight>1]:selected',
             'css': {
                 'background-color': 'yellow',
                 'line-color': 'red',
                 'label': 'data(label)',
                 'target-arrow-color': 'black',
                 'source-arrow-color': 'black',
                 'width': 'data(width)',
             },
             }
        ]
}



# -*- coding: utf-8 -*-
"""
Mapping between data identifiers
"""

import networkx as nx

from magine.mappings.chemical_mapper import ChemicalMapper
from magine.mappings.gene_mapper import GeneMapper

try:
    import cPickle as pickle
except ImportError:
    import pickle as pickle

gm = GeneMapper()
cm = ChemicalMapper()


def convert_all(network, species='hsa', verbose=False):
    """
    Maps gene names to HGNC and kegg compound to HMDB

    Parameters
    ----------
    network : networkx.DiGraph
        network to convert mappings
    species : str
        species of network (hsa, mmu)
    verbose : bool

    Returns
    -------

    """

    change_dict = dict()
    renamed_network = network.copy()
    # fix kegg drugs to their compounds (if possible)

    new_dict = drug_nodes(renamed_network)
    renamed_network = nx.relabel_nodes(renamed_network, new_dict)

    if verbose:
        print('Started converting kegg compounds to HMDB')
    hmdb, kegg_short, chem_names = cm.convert_kegg_nodes(renamed_network)
    nx.set_node_attributes(renamed_network, kegg_short, 'keggName')
    nx.set_node_attributes(renamed_network, chem_names, 'chemName')
    print(chem_names)
    nx.set_node_attributes(renamed_network, hmdb, 'hmdbNames')
    change_dict.update(hmdb)

    if verbose:
        print('Started converting kegg genes to HGNC')
    dict2, kegg_short = gm.convert_kegg_nodes(renamed_network, species=species)
    nx.set_node_attributes(renamed_network, kegg_short, 'keggName')
    change_dict.update(dict2)
    renamed_network = nx.relabel_nodes(renamed_network, change_dict)

    return renamed_network


def drug_nodes(network):
    drug_dict = {}
    hits = set(i for i in network.nodes if i.startswith('dr'))
    for i in hits:
        split_name = i.split(' ')
        if len(split_name) > 1:
            if split_name[1].startswith('cpd:'):
                drug_dict[i] = split_name[1]
                network.node[i]['drug'] = split_name[0]
    return drug_dict


def _check_dict_for_int(dic):
    new_dic = dict()
    for key, value in dic.items():
        if isinstance(value, float):
            continue
        else:
            new_dic[key] = value
    return new_dic



from magine.networks.databases.biogrid_interactions import download_biogrid
from magine.networks.databases.hmdb import load_hmdb_network
from magine.networks.databases.kegg_kgml import download_kegg
from magine.networks.databases.reactome_functional_interaction import \
    download_reactome_fi
from magine.networks.databases.signor import download_signor

if __name__ == '__main__':
    download_reactome_fi()
    download_signor()
    download_biogrid()
    download_kegg()
    load_hmdb_network(fresh_download=True)



import pandas as pd
import numpy as np


def subset_data():
    df = pd.read_csv('reformatted_data.csv.gz', low_memory=False)

    print(df.dtypes)
    genes = pd.pivot_table(df, columns='time_points',
                           index=('gene', 'protein'))

    # genes = genes[genes['time'][1].isnull()]
    genes = genes[genes['time'][6].isnull()]
    genes = genes[genes['time'][24].isnull()]
    list_of_index = genes.index.values
    d = np.array(list_of_index)
    list_of_good = []
    letters = ['B', 'D', 'Z', 'X', 'Y', 'N', 'M']
    for i in d:
        gene_name = i[0]
        cont = False
        for i in letters:
            if i in gene_name:
                cont = True
                continue
        if not cont:
            list_of_good.append(gene_name)
    print(len(list_of_good))
    print(df.shape)
    df = df[df['gene'].isin(list_of_good)]
    print(df.shape)
    df.to_csv('large_example.csv', index=False)

    print(genes.head(10))

    compounds = pd.pivot_table(df, columns='time_points',
                               index=('compound', 'name'))

    print(compounds.shape)
    print(compounds.head(10))



import numpy as np
import pandas as pd

from magine.plotting.heatmaps import heatmap_from_array

flag = 'significant'


class BaseData(pd.DataFrame):
    """
    This class derived from pd.DataFrame
    """
    _index = None

    def __init__(self, *args, **kwargs):
        super(BaseData, self).__init__(*args, **kwargs)

    @property
    def _constructor(self):
        return BaseData

    @property
    def sig(self):
        """ terms with significant flag """
        return self.loc[self[flag]].copy()

    def pivoter(self, convert_to_log=False, columns='sample_id',
                values='fold_change', index=None, fill_value=None, min_sig=0):
        """ Pivot data on provided axis.

        Parameters
        ----------
        convert_to_log : bool
            Convert values column to log2
        index : str
            Index for pivot table
        columns : str
            Columns to pivot
        values : str
            Values of pivot table
        fill_value : float, optional
            Fill pivot table nans with
        min_sig : int
            Required number of significant terms to keep in a row, default 0

        Returns
        -------

        """
        d_copy = self.copy()
        if index is None:
            index = self._index

        if convert_to_log:
            d_copy.log2_normalize_df(values, inplace=True)

        if min_sig:
            if not isinstance(min_sig, int):
                raise AssertionError()
            if 'significant' not in d_copy.columns:
                print('In order to filter based on minimum sig figs, '
                      'please add a "significant" column')

            d_copy.require_n_sig(index=index, columns=columns,
                                 n_sig=min_sig,
                                 inplace=True)
            if not d_copy.shape[0]:
                return pd.DataFrame()

        array = pd.pivot_table(d_copy, index=index, fill_value=fill_value,
                               columns=columns, values=values)
        if isinstance(values, list):
            return array
        if isinstance(columns, list):
            array.sort_values(
                by=sorted(tuple(map(tuple, d_copy[columns].values))),
                ascending=False, inplace=True
            )
        elif isinstance(columns, str):
            array.sort_values(by=sorted(d_copy[columns].unique()),
                              ascending=False, inplace=True)
        return array

    def require_n_sig(self, columns='sample_id', index=None,
                      n_sig=3, inplace=False,
                      verbose=False):
        """ Filter index to have at least "min_terms" significant species.

        Parameters
        ----------
        columns : str
            Columns to consider
        index : str, list
            The column with which to filter by counts
        n_sig : int
            Number of terms required to not be filtered
        inplace : bool
            Filter in place or return a copy of the filtered data
        verbose : bool

        Returns
        -------
        new_data : BaseData
        """
        if index is None:
            index = self._index
        # create safe copy of array
        new_data = self.copy()

        # get list of columns
        cols_to_check = list(new_data[columns].unique())
        if flag not in new_data.columns:
            raise AssertionError('Requires significant column')
        # pivot
        sig = pd.pivot_table(new_data,
                             index=index,
                             fill_value=0,
                             values=flag,
                             columns=columns
                             )[cols_to_check]

        # convert everything that's not 0 to 1
        sig[sig > 0] = 1
        sig = sig[sig.T.sum() >= n_sig]
        if isinstance(index, list):
            keepers = {i[0] for i in sig.index.values}
            new_data = new_data[new_data[index[0]].isin(keepers)]
        elif isinstance(index, str):
            n_before = len(new_data[index].unique())
            keepers = {i for i in sig.index.values}
            new_data = new_data.loc[new_data[index].isin(keepers)]
            n_after = len(new_data[index].unique())
            if verbose:
                print("Number in index went from {} to {}"
                      "".format(n_before, n_after))
        else:
            print("Index is not a str or a list. What is it?")

        if inplace:
            self._update_inplace(new_data)
        else:
            return new_data

    def present_in_all_columns(self, columns='sample_id',
                               index=None, inplace=False):
        """ Require index to be present in all columns

        Parameters
        ----------
        columns : str
            Columns to consider
        index : str, list
            The column with which to filter by counts
        inplace : bool
            Filter in place or return a copy of the filtered data

        Returns
        -------
        new_data : BaseData
        """
        if index is None:
            index = self._index
        # create safe copy of array
        new_data = self.copy()
        n_before = len(new_data[index].unique())
        # get list of columns
        cols_to_check = list(new_data[columns].unique())

        if flag not in new_data.columns:
            raise AssertionError("Missing {} column in data".format(flag))

        # pivot
        pivoted_df = pd.pivot_table(new_data, index=index, fill_value=np.nan,
                                    values=flag, columns=columns
                                    )[cols_to_check]

        # sig = pivoted_df.loc[~np.any(np.isnan(pivoted_df.values), axis=1)]
        sig = pivoted_df.loc[~pivoted_df.isnull().T.any()]
        if isinstance(index, list):
            keepers = {i[0] for i in sig.index.values}
            new_data = new_data[new_data[index[0]].isin(keepers)]
        elif isinstance(index, str):
            keepers = {i for i in sig.index.values}
            new_data = new_data.loc[new_data[index].isin(keepers)]
        else:
            print("Index is not a str or a list. What is it?")
        n_after = len(new_data[index].unique())

        print("Number in index went from {} to {}".format(n_before, n_after))

        if inplace:
            self._update_inplace(new_data)
        else:
            return new_data

    def log2_normalize_df(self, column='fold_change', inplace=False):
        """ Convert "fold_change" column to log2.

        Does so by taking log2 of all positive values and -log2 of all negative
        values.

        Parameters
        ----------
        column : str
            Column to convert
        inplace : bool
            Where to apply log2 in place or return new dataframe

        Returns
        -------

        """
        new_data = self.copy()
        greater = new_data[column] > 0
        less = new_data[column] < 0
        new_data.loc[greater, column] = np.log2(new_data[greater][column])
        new_data.loc[less, column] = -np.log2(-new_data[less][column])
        if inplace:
            self._update_inplace(new_data)
        else:
            return new_data

    def heatmap(self, subset=None, subset_index=None,
                convert_to_log=True, y_tick_labels='auto',
                cluster_row=False, cluster_col=False, cluster_by_set=False,
                index=None, values=None, columns=None,
                annotate_sig=True, figsize=(8, 12), div_colors=True,
                linewidths=0, num_colors=21, sort_row=None, min_sig=0,
                rank_index=None):
        """ Creates heatmap of data, providing pivot and formatting.

        Parameters
        ----------
        subset : list or str
            Will filter to only contain a provided list.
            If a str, will filter based on .contains(subset)
        subset_index : str
            Index to for subset list to match against
        convert_to_log : bool
            Convert values to log2 scale
        y_tick_labels : str
            Column of values, default = 'auto'
        cluster_row : bool
        cluster_col : bool
        cluster_by_set : bool
            Clusters by gene set, only used in EnrichmentResult derived class
        index : str
            Index of heatmap, will be 'row' variables
        values : str
            Values to display in heatmap
        columns : str
            Value that will be used as columns
        annotate_sig : bool
            Add '+' annotation to not 'significant=True' column
        figsize : tuple
            Figure size to pass to matplotlib
        div_colors : bool
            Use colors that are divergent
            (red to blue, instead of shades of blue)
        num_colors : int
            How many colors to include on color bar
        linewidths : float
            line width between individual cols and rows
        sort_row : str
            Rank by 'mean', 'max', 'min' or 'index'
        min_sig : int
            Minimum number of significant 'index' across samples. Can be used to
            remove rows that are not significant across any sample.
        rank_index : bool
            Deprecated, please use sort_row='index' to sort by alphabetically
        Returns
        -------
        matplotlib.figure

        """
        if rank_index is not None:
            raise DeprecationWarning("Please use sort_row='index'")
        if index is None:
            index = self._identifier
        if values is None:
            values = self._value_name
        if columns is None:
            columns = self._sample_id_name
        df = self.copy()
        if subset is not None:
            if subset_index is None:
                subset_index = index
            if isinstance(subset, str):
                df = df.loc[df[subset_index].str.contains(subset)]
            else:
                df = df.loc[df[subset_index].isin(subset)]
        if not df.shape[0]:
            print("No terms match subset")
            return
        return heatmap_from_array(
            df, convert_to_log=convert_to_log,
            y_tick_labels=y_tick_labels,
            cluster_row=cluster_row, cluster_col=cluster_col,
            cluster_by_set=cluster_by_set, figsize=figsize,
            columns=columns, index=index, values=values,
            div_colors=div_colors, num_colors=num_colors,
            sort_row=sort_row, annotate_sig=annotate_sig,
            linewidths=linewidths, min_sig=min_sig, rank_index=rank_index
        )



import os
import shutil
import tempfile

import matplotlib.pyplot as plt
import pandas as pd

import magine.plotting.wordcloud_tools as wt
from magine.enrichment import load_enrichment_csv
from magine.plotting.venn_diagram_maker import create_venn3, create_venn2


class TestVennDiagram(object):
    def setUp(self):
        self.x = ['A', 'B', 'C', 'D']
        self.y = ['C', 'D', 'E', 'F']
        self.z = ['D', 'E', 'F', 'N', 'Z', 'A']
        self.out_dir = tempfile.mkdtemp()

    def tearDown(self):
        self.exp_data = None
        shutil.rmtree(self.out_dir)

    def test_venn_2(self):
        create_venn2(self.x, self.y, 'X', 'Y',
                     os.path.join(self.out_dir, 'test_1'))
        plt.close()

    def test_venn_3(self):
        create_venn3(self.x, self.y, self.z, 'X', 'Y', 'z',
                     os.path.join(self.out_dir, 'test_1'))
        plt.close()


def test_filter():
    df = load_enrichment_csv(os.path.join(os.path.dirname(__file__),
                                          'Data', 'enrichr_test_enrichr.csv'))
    df2 = df.copy()
    df['db'] = 'kegg'
    df2['db'] = 'reactome'
    df2['sample_id'] = 2

    df = pd.concat([df, df2])

    x = wt.create_wordcloud(df)
    out_dir = tempfile.mkdtemp()

    x.plot(save_name=os.path.join(out_dir, 'test_wc'))
    plt.close()
    wt.word_cloud_from_array(df, sample_ids=[1, 2])
    plt.close()



'''


# import pathos.multiprocessing as mp
import time
from itertools import combinations

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from matplotlib.ticker import FormatStrFormatter
from scipy import ndimage
from scipy.stats import chi2_contingency
from scipy.stats.stats import spearmanr
from statsmodels.sandbox.stats.multicomp import multipletests

np.set_printoptions(linewidth=300)


global_data = None
samples = None
import scipy.special as special
from scipy.stats.stats import distributions

from numba import float64, jit
import bottleneck as bn

@jit((float64, float64, float64), nopython=True, nogil=True)
def calc_prob(n, dsq, df):
    denom = n * (n ** 2 - 1) / 6.
    rho = 1. - dsq / denom
    val = (rho + 1.0) * (1.0 - rho)
    if val == 0.0:
        return rho, np.inf
    d = df / val
    return rho, d


@jit(float64(float64, float64), nopython=True, nogil=True)
def calc_val_2(df, t):
    return df / (df + t * t)


def spearmanr_optimized(x, y):
    n = x.size
    xy = x * y
    nans = np.isnan(xy)
    x[nans] = np.nan
    y[nans] = np.nan
    n -= nans.sum()
    df = n - 2
    if df < 0:
        raise ValueError("The input must have at least 3 entries!")

    # Gets the ranks and rank differences
    rankx2 = bn.nanrankdata(x)
    ranky2 = bn.nanrankdata(y)
    dsq = bn.nansum((rankx2 - ranky2) ** 2)
    rho, d = calc_prob(n, dsq, df)
    if d == np.inf or d < 0:
        prob = 0
    else:
        t = np.sqrt(d) * rho
        value_2 = calc_val_2(df, t)
        prob = special.betainc(0.5 * df, 0.5, value_2)
    return rho, prob


def calculate_spearmanr(a, b):
    # new code
    # return spearmanr_optimized(a, b)
    if bn.anynan(a):
        return spearmanr_optimized(a, b)
    elif bn.anynan(b):
        return spearmanr_optimized(a, b)
    ar = np.apply_along_axis(bn.rankdata, 0, a)
    br = np.apply_along_axis(bn.rankdata, 0, b)
    n = a.shape[0]
    rs = np.corrcoef(ar, br, rowvar=0)
    olderr = np.seterr(divide='ignore')  # rs can have elements equal to 1
    try:
        # clip the small negative values possibly caused by rounding
        # errors before taking the square root
        t = rs * np.sqrt(((n - 2) / ((rs + 1.0) * (1.0 - rs))).clip(0))
    finally:
        np.seterr(**olderr)

    prob = 2 * distributions.t.sf(np.abs(t), n - 2)

    if rs.shape == (2, 2):
        return rs[1, 0], prob[1, 0]
    else:
        return rs, prob

def calculate_spearman(pos):
    tmp = global_data[samples[pos, :], :]
    n_nans = np.isfinite(tmp[0, :] * tmp[1, :]).sum()

    if n_nans > 2:
        # cov_v, p_value = calculate_spearmanr(tmp[0, :], tmp[1, :])
        cov_v, p_value = spearmanr(tmp[0, :], tmp[1, :], nan_policy='omit')
        return cov_v, p_value
    else:
        return np.nan, np.nan


EPS = np.finfo(float).eps


def mi_1(d):
    """
    Computes (normalized) mutual information between two 1D variate from a
    joint histogram.
    Parameters
    ----------
    x : 1D array
        first variable
    y : 1D array
        second variable
    sigma: float
        sigma for Gaussian smoothing of the joint histogram
    Returns
    -------
    nmi: float
        the computed similariy measure
    """
    bins = (16, 16)
    x, y, _ = d
    sigma = 1
    normalized = False
    jh = np.histogram2d(x, y, bins=bins)[0]

    # smooth the jh with a gaussian filter of given sigma
    ndimage.gaussian_filter(jh, sigma=sigma, mode='constant',
                            output=jh)

    # compute marginal histograms
    jh = jh + EPS
    sh = np.sum(jh)
    jh = jh / sh
    s1 = np.sum(jh, axis=0).reshape((-1, jh.shape[0]))
    s2 = np.sum(jh, axis=1).reshape((jh.shape[1], -1))

    # Normalised Mutual Information of:
    # Studholme,  jhill & jhawkes (1998).
    # "A normalized entropy measure of 3-D medical image alignment".
    # in Proc. Medical Imaging 1998, vol. 3338, San Diego, CA, pp. 132-143.
    if normalized:
        mi = ((np.sum(s1 * np.log(s1)) + np.sum(s2 * np.log(s2)))
              / np.sum(jh * np.log(jh))) - 1
    else:
        mi = (np.sum(jh * np.log(jh)) - np.sum(s1 * np.log(s1))
              - np.sum(s2 * np.log(s2)))

    return mi


def mi_2(d):
    x, y, bins = d
    c_xy = np.histogram2d(x, y, bins)[0]
    g, p, dof, expected = chi2_contingency(c_xy, lambda_="log-likelihood")
    mi = 0.5 * g / c_xy.sum()
    print(mi)
    return mi


def mi_3(d):
    x, y, bins = d
    c_xy = np.histogram2d(x, y, bins)[0]
    c_x = np.histogram(x, bins)[0]
    c_y = np.histogram(y, bins)[0]

    h_x = shan_entropy(c_x)
    h_y = shan_entropy(c_y)
    h_xy = shan_entropy(c_xy)

    mi = h_x + h_y - h_xy
    return mi


def shan_entropy(c):
    c_normalized = c / float(np.sum(c))
    c_normalized = c_normalized[np.nonzero(c_normalized)]
    return -sum(c_normalized * np.log2(c_normalized))


def correlation_sampling(data, names, save_name, create_plots=True, pool=None):
    global global_data
    global_data = data
    total_values = len(global_data)

    # Create pairwise samples
    global samples

    samples = np.array(list(combinations(range(len(data)), 2)))
    n_samples = int(total_values * (total_values - 1) / 2)
    print("Sampling all = {}".format(n_samples))
    print('Starting to calculate spearman correlations')
    samples_range = range(0, n_samples)
    start_time = time.time()

    all_sample = []
    n_bins = 3
    for i, j in list(combinations(range(len(data)), 2)):
        all_sample.append([data[i], data[j], n_bins])

    # all_sample = [[data[i], data[j], n_bins] for i, j in
    #               combinations(range(len(data)), 2)]

    if pool is None:
        print("Running in serial")
        # x = map(calculate_spearman, samples_range)
        x = map(mi_1, all_sample)
        # x = map(calc_MI, all_sample)

    else:
        print("Running in parallel")
        # x = pool.map(calculate_spearman, samples_range)
        # x = pool.map(calc_mi, all_sample)
        x = pool.map(mi_1, all_sample)
        pool.close()
        pool.join()
    time_taken = time.time() - start_time
    print('Done with {} samples in {}'.format(n_samples, time_taken))
    x = np.array(x)
    use_mi = True

    if use_mi:
        output = pd.DataFrame()
        output['species_1'] = names[samples[:, 0]]
        output['species_2'] = names[samples[:, 1]]
        print(np.shape(x))
        print(output.shape)
        output['mi'] = x
        print(output.mi.min())
        print(output.mi.max())
        plt.hist(output.mi)
        plt.show()
        quit()
    else:
        co_var = x[:, 0]
        pvals = x[:, 1]
        nan_index = np.isfinite(pvals)
        co_var = co_var[nan_index]
        pvals = pvals[nan_index]

        samples = samples[nan_index]

        # Create output file
        output = pd.DataFrame()
        output['species_1'] = names[samples[:, 0]]
        output['species_2'] = names[samples[:, 1]]

        # adjust pvalues for multiple hypothesis testing
        adj_pvalues = multipletests(pvals=pvals, method='fdr_bh')

        output['adj_pvalue'] = adj_pvalues[1]
        output['co_var'] = co_var
        output.to_csv(
            '{}_correlation_sampling_output.csv.gz'.format(save_name),
            index=False, compression='gzip')
    # print('Saved file!')

    if create_plots:
        print("Plotting output")
        fig = plt.figure(figsize=(8, 4))
        bins = np.linspace(-1, 1, 41)
        ax1 = fig.add_subplot(121)
        ax1.hist(np.array(output.co_var), bins=bins)

        output2 = output[output['adj_pvalue'] <= 0.05]
        ax1.hist(np.array(output2.co_var), bins=bins, color='red')
        ax1.set_xlim(-1.1, 1.1)
        plt.xlabel('Correlation', fontsize=20)
        start, end = ax1.get_ylim()
        ax1.yaxis.set_ticks(np.linspace(start, end, 4))

        ax2 = fig.add_subplot(122)
        bins = np.linspace(0, 1, 21)
        ax2.hist(np.array(output.adj_pvalue), bins=bins)
        ax2.hist(np.array(output2.adj_pvalue), bins=bins, color='red')
        ax2.set_xlim(0, 1)
        plt.xlabel('Adjusted P value', fontsize=20)

        ax1.yaxis.set_major_formatter(FormatStrFormatter('%.1g'))
        ax2.yaxis.set_major_formatter(FormatStrFormatter('%.1g'))
        start, end = ax2.get_ylim()
        ax2.yaxis.set_ticks(np.linspace(start, end, 4))
        ax1.tick_params(axis='y', labelsize=16)
        ax1.tick_params(axis='x', labelsize=16)
        ax2.tick_params(axis='y', labelsize=16)
        ax2.tick_params(axis='x', labelsize=16)
        plt.tight_layout(h_pad=0.15)
        _same_name = '{}_correlation_and_pvalue_histogram.png'.format(
            save_name)
        print("Saving to  {}".format(_same_name))
        plt.savefig(_same_name, bbox_inches='tight')
        plt.close()
        print("Done 2")
    return output
'''



import itertools
import os

import networkx as nx
import pandas as pd
from goatools import obo_parser
from goatools.semantic import TermCounts, ic, resnik_sim, semantic_similarity
from magine.enrichment.ontology_analysis import MagineGO

from magine.data.storage import id_mapping_dir

obo_file = os.path.join(id_mapping_dir, 'go.obo')

if not os.path.exists(obo_file):
    print("Using ontology for first time")
    print("Downloading files")
    from magine.enrichment.databases.gene_ontology import \
        download_and_process_go
    download_and_process_go()
    assert os.path.exists(obo_file)

go = obo_parser.GODag(obo_file)

mg = MagineGO()
print("Loading termcounts")
associations = mg.gene_to_go
termcounts = TermCounts(go, associations)
print("Loaded termcounts")


def path_to_root(go_term):
    """
    Creates networkx graph from provided term to root term

    Parameters
    ----------
    go_term : str
        source GO term

    Returns
    -------
    graph : nx.DiGraph
        Graph containing all paths from provided GO term to root

    """

    graph = nx.DiGraph()
    paths = go.paths_to_top(go_term)
    all_terms = set()
    for i in paths:
        for j in i:
            all_terms.add(j.id)
    for i in paths:
        for j in i:
            add_children(j.id, graph, all_terms)
    return graph


def print_path_to_root(go_term):
    paths = go.paths_to_top(go_term)
    all_terms = set()
    for i in paths:
        print('\n')
        for j in i:
            all_terms.add(j.id)
            print("GO id = {}, GO name = {}".format(j.id, j.name))


def print_children(go_term):
    print("({}) {} has children of :".format(go[go_term].id, go[go_term].name))
    for i in go[go_term].children:
        print("\t({}) {}".format(i.id, i.name))


def common_parent_go_ids(terms):
    """ Find common parents of list of GO terms

    Parameters
    ----------
    terms: list
    """
    # Find candidates from first
    terms = list(terms)
    path_parents = go[terms[0]]
    candidates = path_parents.get_all_parents()
    candidates.update({terms[0]})

    # Find intersection with second to nth term
    for term in terms[1:]:
        path_parents = go[term]
        parents = path_parents.get_all_parents()
        parents.update({term})

        # Find the intersection with the candidates, and update.
        candidates.intersection_update(parents)

    return candidates


def deepest_common_ancestor(terms):
    """
        This function gets the nearest common ancestor
        using the above function.
        Only returns single most specific - assumes unique exists.
    """
    # Take the element at maximum depth.
    return max(common_parent_go_ids(terms), key=lambda t: go[t].depth)


def min_branch_length(go_id1, go_id2):
    """
        Finds the minimum branch length between two terms in the GO DAG.
    """
    # First get the deepest common ancestor
    dca = deepest_common_ancestor([go_id1, go_id2])

    # Then get the distance from the DCA to each term
    dca_depth = go[dca].depth
    d1 = go[go_id1].depth - dca_depth
    d2 = go[go_id2].depth - dca_depth
    # print(d1, d2)
    # Return the distance from one term to the deepest common ancestor and
    # back to the other term.
    return d1 + d2


def add_children(go_term, graph, gene_set_of_interest):
    """
    add children to graph of provided GO term

    Parameters
    ----------
    go_term : str
        parent GO term
    graph : networkx.DiGraph
        graph to add children
    gene_set_of_interest : set
        all terms to consider

    Returns
    -------

    """
    nodes_in_graph = set(graph.nodes())
    graph.add_node(go[go_term].id,
                   depth=go[go_term].depth,
                   level=go[go_term].level,
                   GOname=go[go_term].name,
                   ic=ic(go[go_term].id, termcounts))
    for i in go[go_term].children:
        if i.id in gene_set_of_interest:
            if i.id not in nodes_in_graph:
                graph.add_node(i.id,
                               depth=i.depth,
                               level=i.level,
                               GOname=i.name,
                               ic=ic(i.id, termcounts))
            graph.add_edge(go[go_term].id, i.id)
            add_children(i.id, graph, gene_set_of_interest)


def check_if_children(list_of_terms, verbose=False):
    """
    Checks to see if any term is a child of another


    Parameters
    ----------
    list_of_terms : list
        list of all terms to consider
    verbose : bool

    Returns
    -------

    """
    list_of_terms = set(list_of_terms)
    to_remove = set()
    for i in list_of_terms:
        g = path_to_root(i)
        nodes = set(g.nodes())
        for j in list_of_terms:
            if not i == j:
                if j in nodes:
                    if verbose:
                        print("{} is a parent of {}, "
                              "removing from list".format(i, j))
                    to_remove.add(i)
    return list_of_terms.difference(to_remove)


def check_depth_level(list_of_terms, verbose=False):
    """
    Checks to see if any terms are too deep in graph

    Parameters
    ----------
    list_of_terms : list
        list of GO terms
    verbose : bool


    Returns
    -------

    """
    list_of_terms = set(list_of_terms)
    to_remove = set()

    for i in list_of_terms:
        if go[i].depth > 11:
            if verbose:
                print("Depth of {} is greater than 11.".format(i),
                      "Removing from list for now as path to root is VAST")
            to_remove.add(i)
    return list_of_terms.difference(to_remove)


def check_depth_and_children(list_of_terms):
    set_of_terms = set(list_of_terms)
    return check_if_children(check_depth_level(set_of_terms))


def create_graph_to_root_from_list_terms(list_of_terms, check_child=False,
                                         check_depth=True):
    """ calculate a path to root for a list of terms

    Parameters
    ----------
    list_of_terms: list
        list of GO terms
    check_child: bool
        if you want to check if there are terms that are children of other
        terms within your list
    check_depth: bool
        check for maximum depth value. Depth larger than 10 usually
        takes a long time to find all paths.


    Returns
    -------
    networkx.DiGraph

    """
    if check_child:
        list_of_terms = check_if_children(list_of_terms)
    if check_depth:
        list_of_terms = check_if_children(list_of_terms)
    combined_graph = nx.DiGraph()
    for i in list_of_terms:
        g = path_to_root(i)
        comb_nodes = set(combined_graph.nodes())
        combined_graph.add_nodes_from(g.nodes(data=True))
        combined_graph.add_edges_from(g.edges(data=True))
        for n in g.nodes():
            if n in comb_nodes:
                combined_graph.node[n]['counter'] += 1
            else:
                combined_graph.node[n]['counter'] = 1
    for each in list_of_terms:
        combined_graph.add_node(each, style='filled', fillcolor='green',
                                **combined_graph[each])

    return combined_graph


def check_term_list(list_of_terms, verbose=False):
    """

    Parameters
    ----------
    list_of_terms : list
    verbose : bool

    Returns
    -------

    """
    list_of_terms = check_depth_and_children(list_of_terms)
    combos = itertools.combinations(list_of_terms, 2)
    to_remove = set()
    for i, j in combos:

        sim2 = semantic_similarity(i, j, go)
        dca = deepest_common_ancestor([i, j])

        sim = resnik_sim(i, j, go, termcounts)
        ic_i = ic(i, termcounts)
        ic_j = ic(j, termcounts)
        ic_dca = ic(dca, termcounts)

        if verbose:
            print("\nGO 1\t\t GO 2\t\t GO3")
            print("Min branch length = {}".format(min_branch_length(i, j)))
            print("{}\t{}\t{}".format(go[i].name, go[j].name, go[dca].name))
            print("{}\t{}\t{}".format(i, j, dca))
            print("{}\t{}\t{}".format(go[i].depth, go[j].depth, go[dca].depth))
            print("Resnik {}\tSemantic {}".format(sim, sim2))
            # print("{}\t{}\n".format(sim, sim2))

            print("IC\t\t{}\t\t\t{}\t\t\t{}".format(ic_i, ic_j, ic_dca))

        # remove if two terms of similar by distance in graph
        if sim2 > .5:
            if go[i].depth > go[j].depth:
                to_remove.add(i)
            else:
                to_remove.add(j)
        # remove if deepest common ancestor has at least 4 IC
        elif sim > 9:
            if go[dca].depth < 3:
                continue
            list_of_terms.add(dca)
            to_remove.add(i)
            to_remove.add(j)
            if dca in to_remove or dca in list_of_terms:
                list_of_terms.add(dca)
                to_remove.add(i)
                to_remove.add(j)
    if verbose:
        print(to_remove)
        print('Number of term before = {}'.format(len(list_of_terms)))

    list_of_terms = list_of_terms.difference(to_remove)

    if verbose:
        print('Number of term after = {}'.format(len(list_of_terms)))

    return list_of_terms


def find_disjunction_common_ancestor(list_of_terms):
    """

    Parameters
    ----------
    list_of_terms : list_like
        list of GO terms

    Returns
    -------
    graph : networkx.DiGraph
        containing sources to root, disjunction common ancestor show in red
    """
    list_of_terms = check_depth_and_children(list_of_terms)
    combined_graph = create_graph_to_root_from_list_terms(list_of_terms)

    # dca = deepest_common_ancestor(list(list_of_terms))

    # find the nodes that are in all terms
    n_terms = len(list_of_terms)
    # print("N terms", n_terms)
    term_counter = nx.get_node_attributes(combined_graph, 'counter')
    common_nodes = set()
    for i in term_counter:
        # print(i, term_counter[i], go[i.replace('"', '')].depth)
        if term_counter[i] == n_terms:
            common_nodes.add(i)

    if len(common_nodes) == 1:
        node_0 = combined_graph.nodes()[0]
        level = combined_graph.node[node_0]['level']
        depth = combined_graph.node[node_0]['depth']
        name = combined_graph.node[node_0]['GOname']
        print("Warning : the most common ancestor is {} ({}) with a depth of "
              "{} and level of {}".format(name, node_0, depth, level))

    # Check to see if any of the provided terms are in all graphs
    # This means it was an ancestor node and should be removed from list
    # to_remove = []
    # for each in list_of_terms:
    #     if each in common_nodes:
    #         print("Warning : {} term is already in graph! Thus will treat as "
    #               "upstream term and remove from list of terms"
    #               " provided".format(each))
    #         to_remove.append(each)
    # for each in to_remove:
    #     list_of_terms.remove(each)
    print(common_nodes)
    # return subgraph containing only nodes that are in all
    common_graph = combined_graph.subgraph(common_nodes)

    # find the common dijunction nodes and shortest paths between
    disjunction_nodes = set()
    for i in common_graph.nodes():
        if common_graph.out_degree(i) == 0:
            disjunction_nodes.add(i)

    # color disjunction nodes and find all shortest paths between source nodes
    # and each disjunction node
    for disjun_node in disjunction_nodes:
        common_graph.node[disjun_node]['style'] = 'filled'
        common_graph.node[disjun_node]['fillcolor'] = 'red'
        for each in list_of_terms:
            if nx.has_path(combined_graph, disjun_node, each):
                print("From {} to {}".format(disjun_node, each))
                sh_paths = [p for p in
                            nx.all_shortest_paths(combined_graph, disjun_node,
                                                  each)]

                # ic_paths = []
                # for path in sh_paths:
                #     ic_path = 0
                #     for g in path:
                #         ic_local = combined_graph.node[g]['ic']
                #         ic_path += ic_local
                #     ic_paths.append(ic_path)
                # min_ic = max(ic_paths)
                for n, path in enumerate(sh_paths):
                    # if ic_paths[n] == min_ic:
                    #     for g in path:
                    #         common_graph.add_node(g, **combined_graph.node[g])
                    #     common_graph.add_path(path)
                    for g in path:
                        common_graph.add_node(g, **combined_graph.node[g])
                    common_graph.add_path(path)

    # color source nodes
    for each in list_of_terms:
        common_graph.add_node(each, **combined_graph.node[each])

    return common_graph


def filter_ontology_df(data, n_hits_per_time=None, go_aspects=None,
                       trim_nodes=False, additional_ids_to_include=None,
                       pvalue=0.05, min_depth=3, max_depth=10, min_ref=5,
                       max_ref=500, variable_of_interest='enrichment_score'):
    """

    Parameters
    ----------
    data :  pandas.DataFrame 
        output from magine.ontology_analysis.calculate_enrichment
    trim_nodes : bool, optional, default=False
        remove GO terms that are similar
    n_hits_per_time : int, optional
        number of terms for each sample
    go_aspects : list, optional
        aspects to plots, options are
        {"biological_process", "cellular_compartment", "molecular_function"}
    additional_ids_to_include: list
        list of additional GO ids to keep
    pvalue : float
        upper pvalue limit to filter terms
    min_depth : int
        minimum depth of GO ontology
    max_depth : int
        maximum depth of GO ontology
    min_ref : int
        minimum number of references for GO ontology term
    max_ref : int
        maximum number of references for GO ontology term
    variable_of_interest : str, {"enrichment_score", "pvalue}



    Returns
    -------

    """

    if isinstance(data, str):
        data = pd.read_csv(data)

    # make sure all columns we need are defined
    for i in ['GO_id', 'pvalue', 'enrichment_score', 'sample_index', 'aspect']:
        if i not in data.columns:
            raise AssertionError("Must have {} in df".format(i))

    data['genes'] = data['genes'].astype(set)

    # removes aspects of GO that are not wanted
    if go_aspects is None:
        go_aspects = ['biological_process']
    if isinstance(go_aspects, str):
        data = data[data['aspect'] == go_aspects]
    elif isinstance(go_aspects, list):
        data = data[data['aspect'].isin(go_aspects)]
    else:
        print("go_aspect must be a list! \n"
              "biological_process, cellular_component, molecular_function")

    # filter terms based on reference and depth
    if max_ref is not None:
        data = data[data['ref'] <= max_ref]
    if min_ref is not None:
        data = data[data['ref'] >= min_ref]
    if max_depth is not None:
        data = data[data['depth'] <= max_depth]
    if min_depth is not None:
        data = data[data['depth'] >= min_depth]

    # filter out non-signficant terms
    tmp = data[data['pvalue'] <= pvalue]

    # create sample labels
    labels = tmp['sample_index'].unique()

    enrichment_list = [(variable_of_interest, i) for i in labels]
    ascend = False
    if variable_of_interest == 'pvalue':
        ascend = True

    list_all_go = tmp['GO_id'].unique()
    if n_hits_per_time is not None:
        tmp = pd.pivot_table(tmp, index=['GO_id', ], columns='sample_index')

        def find_n_go_terms(terms, updated_index):
            n_needed = n_hits_per_time - len(terms)
            terms.update(set(list(tmp.index)[:updated_index + n_needed]))
            return check_term_list(terms)

        list_all_go = set()
        for i in enrichment_list:
            tmp = tmp.sort_values(by=i, ascending=ascend)
            list_of_go = set(tmp.head(n_hits_per_time).index)
            if trim_nodes:
                list_of_go = check_term_list(list_of_go)
                count = n_hits_per_time
                while len(list_of_go) < n_hits_per_time:
                    list_of_go = find_n_go_terms(list_of_go, count)
                    count += 1

            list_all_go.update(list_of_go)

    if trim_nodes:
        list_all_go = check_term_list(list_all_go)

    if additional_ids_to_include is not None:
        assert isinstance(additional_ids_to_include, list)
        additional_ids_to_include = set(additional_ids_to_include)
        if n_hits_per_time is not None:
            additional_ids_to_include.update(list_all_go)
        data = data[data['GO_id'].isin(additional_ids_to_include)]

    elif list_all_go is not None:
        data = data[data['GO_id'].isin(list_all_go)]

    return data


def slim_ontology(data, pvalue=0.05, n_top_hits=None, go_aspects=None,
                  trim_nodes=False, additional_ids_to_include=None,
                  min_depth=3, max_depth=10, min_ref=5, max_ref=500,
                  variable_of_interest='enrichment_score'):
    """

    Parameters
    ----------
    data :  pandas.DataFrame 
        output from magine.ontology_analysis.calculate_enrichment
    pvalue : float
        upper pvalue limit to filter terms
    trim_nodes : bool, optional, default=False
        remove GO terms that are similar
    n_top_hits : int, optional
        number of terms for each sample
    go_aspects : list, optional
        aspects to plots, options are
        {"biological_process", "cellular_compartment", "molecular_function"}
    additional_ids_to_include: list
        list of additional GO ids to keep
    min_depth : int
        minimum depth of GO ontology
    max_depth : int
        maximum depth of GO ontology
    min_ref : int
        minimum number of references for GO ontology term
    max_ref : int
        maximum number of references for GO ontology term
    variable_of_interest : str, {"enrichment_score", "pvalue"}
    Returns
    -------

    """

    if isinstance(data, str):
        data = pd.read_csv(data)

    # make sure all columns we need are defined
    for i in ['GO_id', 'pvalue', 'enrichment_score', 'aspect']:
        assert i in data.columns, "Must have {} in df".format(i)

    data['genes'] = data['genes'].astype(set)

    # removes aspects of GO that are not wanted
    if go_aspects is None:
        go_aspects = ['biological_process']
    if isinstance(go_aspects, str):
        data = data[data['aspect'] == go_aspects]
    elif isinstance(go_aspects, list):
        data = data[data['aspect'].isin(go_aspects)]
    else:
        print("go_aspect must be a list! \n"
              "biological_process, cellular_component, molecular_function")

    # remove terms with reference smaller than 5
    if max_ref is not None:
        data = data[data['ref'] <= max_ref]
    if min_ref is not None:
        data = data[data['ref'] >= min_ref]
    if max_depth is not None:
        data = data[data['depth'] <= max_depth]
    if min_depth is not None:
        data = data[data['depth'] >= min_depth]

    # normalize maximum enrichment to 20
    # done for simplicity, may not be desired
    # data.loc[data['enrichment_score'] > 20, 'enrichment_score'] = 20

    # filter out non-significant terms
    tmp = data[data['pvalue'] <= pvalue].copy()
    assert variable_of_interest in ('enrichment_score', 'pvalue'), \
        "variable_of_interest must be 'enrichment_score' or 'pvalue'"

    enrichment_list = [variable_of_interest]

    list_all_go = tmp['GO_id'].unique()
    if n_top_hits is not None:
        def find_n_go_terms(terms, updated_index):
            n_needed = n_top_hits - len(terms)
            terms.update(set(list(tmp.index)[:updated_index + n_needed]))
            return check_term_list(terms)

        list_all_go = set()
        tmp.set_index('GO_id', inplace=True)
        tmp.sort_values(by=enrichment_list, ascending=False, inplace=True)
        list_of_go = set(tmp.head(n_top_hits).index)
        if trim_nodes:
            list_of_go = check_term_list(list_of_go)
            print("start {}", format(list_of_go))
            count = n_top_hits
            while len(list_of_go) < n_top_hits:
                list_of_go = find_n_go_terms(list_of_go, count)
                count += 1
        list_all_go.update(list_of_go)

    if trim_nodes:
        list_all_go = check_term_list(list_all_go)

    if additional_ids_to_include is not None:
        assert isinstance(additional_ids_to_include, list)
        additional_ids_to_include = set(additional_ids_to_include)
        if n_top_hits is not None:
            additional_ids_to_include.update(list_all_go)
        data = data[data['GO_id'].isin(additional_ids_to_include)]

    elif list_all_go is not None:
        data = data[data['GO_id'].isin(list_all_go)]

    return data



from magine.networks.visualization.graphviz import draw_graphviz
from magine.networks.visualization.igraph_viz import draw_igraph
from magine.networks.visualization.mpl import draw_mpl
from magine.networks.visualization.notebooks.view import draw_cyjs

try:
    from magine.networks.visualization.cytoscape import RenderModel
except:
    RenderModel = None

__all__ = ['draw_igraph', 'draw_mpl', 'draw_graphviz', 'draw_cyjs',
           'RenderModel']



import itertools
import operator
from collections import OrderedDict
from itertools import combinations

import numpy as np
import pandas as pd

from magine.data.base import BaseData
from magine.plotting.heatmaps import cluster_distance_mat

# Will be OK in Python 2
try:
    basestring
# Allows isinstance(foo, basestring) to work in Python 3
except:
    basestring = str

sig = 'significant'


def load_enrichment_csv(file_name, **args):
    """ Load data into EnrichmentResult data class

    Parameters
    ----------
    file_name : str

    Returns
    -------
    EnrichmentResult

    """
    d = pd.read_csv(file_name, **args)
    return EnrichmentResult(d)


class EnrichmentResult(BaseData):

    def __init__(self, *args, **kwargs):
        super(EnrichmentResult, self).__init__(*args, **kwargs)
        self._index = 'term_name'
        self._identifier = 'term_name'
        self._value_name = 'combined_score'
        self._sample_id_name = 'sample_id'

    @property
    def _constructor(self):
        return EnrichmentResult

    def filter_rows(self, column, options, inplace=False):
        """
        Filters a pandas dataframe provides a column and filter selection.

        Parameters
        ----------
        column : str
        options : str, list
            Can be a single entry or a list
        inplace : bool
            Filter inplace
        Returns
        -------
        pd.DataFrame
        """

        new_data = self.copy()
        valid_opts = sorted(new_data[column].unique())
        if isinstance(options, str):
            if options not in valid_opts:
                print('{} not in {}'.format(options, valid_opts))
            else:
                new_data = new_data[new_data[column] == options]
        elif isinstance(options, list):
            for i in options:
                if i not in valid_opts:
                    print('{} not in {}'.format(i, valid_opts))
            new_data = new_data[new_data[column].isin(options)]

        if inplace:
            self._update_inplace(new_data)
        else:
            return new_data

    def filter_multi(self, p_value=None, combined_score=None, db=None,
                     sample_id=None, category=None, rank=None, inplace=False):
        """
        Filters an enrichment array.

        This is an aggregate function that allows ones to filter an entire
        dataframe with a single function call.

        Parameters
        ----------
        p_value : float
            filters all values less than or equal
        combined_score : float
            filters all values greater than or equal
        db : str, list
        sample_id : str, list
        category : str, list
        rank : int
        inplace : bool
            Filter inplace

        Returns
        -------
        new_data : EnrichmentResult
        """
        new_data = self.copy()
        if p_value is not None:
            if not isinstance(p_value, (int, float)):
                raise AssertionError("p_value must be a float or int")
            new_data = new_data[new_data['adj_p_value'] <= p_value]
        if combined_score is not None:
            if not isinstance(combined_score, (int, float)):
                raise AssertionError("combined_score must be a float or int")
            new_data = new_data[new_data['combined_score'] >= combined_score]
        if isinstance(rank, (int, float)):
            new_data = new_data[new_data['rank'] <= rank]
        if db is not None:
            new_data.filter_rows('db', db, inplace=True)
        if sample_id is not None:
            new_data.filter_rows('sample_id', sample_id, inplace=True)
        if category is not None:
            new_data.filter_rows('category', category, inplace=True)
        if inplace:
            self._update_inplace(new_data)
        else:
            return new_data

    def term_to_genes(self, term):
        """ Get set of genes of provides term(s)

        Parameters
        ----------
        term : str, list

        Returns
        -------
        set

        """
        if isinstance(term, basestring):
            genes = self[self['term_name'].isin([term])]['genes']
        else:
            genes = self[self['term_name'].isin(term)]['genes']
        return set(itertools.chain.from_iterable(genes.str.split(',').values))

    def term_to_genes_dict(self, term_list=None):
        """

        Parameters
        ----------
        term_list : list

        Returns
        -------
        OrderedDict

        """
        if term_list is None:
            term_list = set(self['term_name'].values)
        elif isinstance(term_list, basestring):
            term_list = [term_list]
        gene_to_term = {}
        for term in term_list:
            gene_list = self.term_to_genes(term)
            for g in gene_list:
                if g not in gene_to_term:
                    gene_to_term[g] = set()
                gene_to_term[g].add(term)
        term_to_gene = {}
        for i, j in gene_to_term.items():
            name = ','.join(sorted(j))
            if name not in term_to_gene:
                term_to_gene[name] = set()
            term_to_gene[name].add(i)
        return OrderedDict(
            sorted(term_to_gene.items(), key=operator.itemgetter(0))
        )

    def all_genes_from_df(self):
        """ Returns all genes from gene columns in a set

        Returns
        -------
        set
        """
        return set(
            itertools.chain.from_iterable(self['genes'].str.split(',').values)
        )

    def filter_based_on_words(self, words, inplace=False):
        """ Filter term_name based on key terms

        Parameters
        ----------
        words : list, str
            List of words to use to keep rows in dataframe
        inplace : bool
            Filter the dataframe in place or return filtered copy

        Returns
        -------
        pandas.DataFrame

        """
        if isinstance(words, str):
            words = [words]
        df = self.copy()
        df = df[df['term_name'].str.lower().str.contains('|'.join(words))]
        if inplace:
            self._update_inplace(df)
        else:
            return df

    def find_similar_terms(self, term, level='sample', remove_subset=True):
        """ Calculates similarity of all other terms to given term

        Parameters
        ----------
        term : str
        level : str
            Sample or dataframe level, flattens all terms to one set of genes
        remove_subset : bool
            If any term is a subset of the other term, a score of 1 will be
            used instead of jaccard index.


        Returns
        -------
        pd.DataFrame
        """

        rest_of_df = self[~(self['term_name'] == term)].copy()
        first_genes = self.term_to_genes(term)

        if level == 'dataframe':
            vals = [[i, self.term_to_genes(i)] for i in
                    rest_of_df['term_name'].unique()]
        else:
            vals = [[i, j.split(',')] for i, j in
                    rest_of_df[['term_name', 'genes']].values]

        dist_m = [[i, jaccard_index(first_genes, j, remove_subset)]
                  for i, j in vals]

        df = pd.DataFrame(dist_m, columns=['term_name', 'similarity_score'])
        df.sort_values('similarity_score', inplace=True, ascending=False)
        return df

    def show_terms_below(self, term, level='dataframe', threshold=.7,
                         remove_subset=True):
        """
        Find terms that were removed by remove_redundant

        Parameters
        ----------
        term : str
        level : str
        threshold : float
        remove_subset : bool

        Returns
        -------
        EnrichmentResult
        """
        temp_df = self.copy()
        # calculate similarity of term to all terms
        sim_terms = temp_df.find_similar_terms(term,
                                               remove_subset=remove_subset,
                                               level=level)
        # gather terms that are highly similar
        sim_terms = sim_terms.loc[sim_terms.similarity_score >= threshold]
        high_similar_terms = set(sim_terms.term_name.values)

        # this shows all terms that remained after filtering
        # Will be used to compare against
        term_kept = temp_df.remove_redundant(
            threshold=threshold,
            level=level,
            inplace=False,
            sort_by='combined_score'
        )

        term_kept = set(term_kept.term_name.values)
        terms_removed = high_similar_terms.difference(term_kept)
        # Adding the original term to show similarity
        terms_removed.add(term)
        return temp_df.loc[temp_df.term_name.isin(terms_removed)]

    def remove_redundant(self, threshold=0.75, verbose=False, level='sample',
                         sort_by='combined_score', inplace=False):
        """
        Calculate similarity between all term sets and removes redundant terms.

        Parameters
        ----------
        threshold : float, default 0.75
        verbose : bool, default False
            Print similarity scores and removed terms.
        level : {'sample', 'dataframe'}, default 'sample'
            Level to filter dataframe. 'sample' will pivot the dataframe and
            filter each group of 'sample_id' individually. 'dataframe' will
            merge all genes that share the same 'term_name'.
        sort_by : {'combined_score', 'rank', 'adj_p_value', 'n_genes'},
            default 'combined_score'
            Keyword to sort the dataframe. The scoring starts at the top term and
            compares to all the lower terms. Options are
        inplace : bool
            Filter the dataframe in place or return filtered copy

        Returns
        -------
        pandas.DataFrame

        """

        if sort_by in ('rank', 'adj_p_value'):
            ascending = True
        else:
            ascending = False

        self.sort_values(sort_by, inplace=True, ascending=ascending)
        data_copy = self.copy()
        if 'sample_id' not in data_copy.columns or level == 'dataframe':
            to_keep = data_copy.unique_terms(threshold, verbose, level=level)
        else:
            to_keep = set()
            for i in sorted(data_copy['sample_id'].unique()):
                tmp = data_copy[data_copy['sample_id'] == i]
                to_keep.update(
                    tmp.unique_terms(threshold, verbose, level=level)
                )

        data_copy = data_copy[(data_copy['term_name'].isin(to_keep))]
        print("Number of rows went from {} to {}"
              "".format(len(self.term_name.unique()),
                        len(data_copy.term_name.unique()))
              )
        if inplace:
            self._update_inplace(data_copy)
        else:
            return data_copy

    def unique_terms(self, threshold=0.75, verbose=False, level='dataframe'):
        """

        Parameters
        ----------
        threshold : float
        verbose : bool
        level : str, {'dataframe', 'each'}

        Returns
        -------

        """
        if level == 'dataframe':
            names = self['term_name'].unique()
            scores = self._get_distance_all()
        else:
            names = self['term_name'].values
            scores = self._get_distance_each()
        to_remove, to_keep = set(), set()

        n_dim = len(names)
        ind = 0
        for i, term_1 in enumerate(names):
            if term_1 not in to_remove:
                to_keep.add(term_1)
            else:
                for j in range(n_dim):
                    if i >= j:
                        continue
                    ind += 1
                continue
            if verbose:
                print("Finding matches for {}".format(term_1))
            for j, term_2 in enumerate(names):
                if i >= j:
                    continue
                score = scores[ind]
                ind += 1
                if score > threshold:
                    to_remove.add(term_2)
                if verbose:
                    print("\tScore for {} is {:.3f}".format(term_2, score))
                    if verbose:
                        print("\t\tRemoving {}".format(term_2))

        return to_keep

    def dist_matrix(self, figsize=(8, 8), level='dataframe'):
        """ Create a distance matrix of all term similarity

        Parameters
        ----------
        figsize : tuple
            Size of figure
        level : str, {'dataframe', 'each'}
            How to treats term_name to genes. Dataframe compresses all genes
            from all sample_ids into same term. 'each' treats each term_name
            individually.

        Returns
        -------
        matplotlib.Figure

        """
        mat, names = self.calc_dist(level)
        return cluster_distance_mat(mat, names, figsize)

    def calc_dist(self, level='datafame'):
        if level == 'each':
            names = self['term_name'].values
            scores = self._get_distance_each()
        else:
            names = self['term_name'].unique()
            scores = self._get_distance_all()
        n_dim = len(names)

        mat = np.ones((n_dim, n_dim), dtype=float)
        ind = 0
        for i in range(n_dim):
            for j in range(n_dim):
                if i >= j:
                    continue
                mat[i, j] = scores[ind]
                mat[j, i] = scores[ind]
                ind += 1
        return mat, names

    def _get_distance_each(self):
        vals = [set(i.split(',')) for i in self['genes'].values]
        return list(map(_score, combinations(vals, 2)))

    def _get_distance_all(self):
        vals = [self.term_to_genes(i) for i in self['term_name'].unique()]
        return list(map(_score, combinations(vals, 2)))


def _score(vals):
    return jaccard_index(vals[0], vals[1])


def jaccard_index(set1, set2, remove_subset=True):
    """
    Computes the similarity between two sets.
        https://en.wikipedia.org/wiki/Jaccard_index

    Parameters
    ----------
    set1 : set
    set2 : set
    remove_subset : bool
        If a set is a subset of the other, return 1.


    Returns
    -------
    index : float


    References
    ----------
    .. [1] `Wikipedia entry for the Jaccard index
           <https://en.wikipedia.org/wiki/Jaccard_index>`_
    """
    union = len(set1.union(set2))
    max_size = max(len(set1), len(set2))
    if union == max_size and remove_subset:
        return 1.
    return float(len(set1.intersection(set2))) / float(union)



import matplotlib.pyplot as plt
from matplotlib_venn import venn2 as _venn2, venn3 as _venn3


def create_venn3(list1, list2, list3, label1, label2, label3, save_name=None,
                 image_format='png', title=None, ax=None,
                 colors=('g', 'r', 'b')):
    """ Creates a venn digram containing for 3 lists
    
    Parameters
    ----------
    list1 : list_like
    list2 : list_like
    list3 : list_like
    label1 : str
    label2 : str
    label3 : str
    save_name : str
    image_format : str
        default png
    title: str
    ax : matplotlib.axes

    Returns
    -------

    """
    set1 = set(list1)
    set2 = set(list2)
    set3 = set(list3)
    if len(colors) != 3:
        raise Exception("Must provide a list of three colors")
    v = _venn3([set1, set2, set3], ('%s(%s)' % (label1, str(len(set1))),
                                    '%s(%s)' % (label2, str(len(set2))),
                                    '%s(%s)' % (label3, str(len(set3)))),
               set_colors=colors,
               ax=ax)
    plt.tight_layout()
    if title is not None:
        plt.title(title)
    if save_name is not None:
        save_name = "{}.{}".format(save_name, image_format)
        plt.savefig(save_name, bbox_inches='tight')
    return v


def create_venn2(list1, list2, label1, label2, save_name=None, title=None,
                 image_format='png', ax=None):
    """Creates a venn digram containing for 2 lists
    
    Parameters
    ----------
    list1 : list_like
    list2 : list_like
    label1 : str
    label2 : str
    save_name : str
    title : str
    image_format : str, optional
        default png
    ax : matplotlib.axes
    
    Returns
    -------
    
    """
    set1 = set(list1)
    set2 = set(list2)
    v = _venn2([set1, set2], ('%s(%s)' % (label1, str(len(set1))),
                              '%s(%s)' % (label2, str(len(set2))),),
               ax=ax)
    plt.tight_layout()
    if title is not None:
        plt.title(title)
    if save_name is not None:
        save_name = "{}.{}".format(save_name, image_format)
        plt.savefig(save_name, bbox_inches='tight')

    return v



"""Interface to mapping between species IDs.

We constructed two classes for mappings ids.
1. GeneMapper
2. ChemicalMapper


Currently supported options
---------------------------

1. Genes
    KEGG, HGNC, Uniprot, Entrez, ensembl_gene_id
2. Metabolites/compounds
    kegg_id, name, accession, chebi_id, chemspider_id, biocyc_id, synonyms,
    pubchem_compound_id, protein_associations, inchikey, iupac_name, ontology,
    drugbank_id, chemical_formula, smiles, metlin_id, average_molecular_weight

"""
from magine.mappings.chemical_mapper import ChemicalMapper
from magine.mappings.gene_mapper import GeneMapper

__all__ = ['ChemicalMapper', 'GeneMapper']




import networkx as nx
from nose.tools import ok_

import magine.networks.utils as utils

"""
def test_paint_over_time():
    graph = nx.DiGraph()
    graph.add_nodes_from(['a', 'b', 'c'])
    labels = ['1', '2', '3']
    colors = ['red', 'green', 'blue']
    nodes = [['a'], ['c', 'b'], ['a', 'b', 'c']]
    utils.paint_network_overtime(graph, nodes, colors,
                                 labels=labels,
                                 save_name='ok',
                                 create_gif=False)


def test_paint_over_time_up_down():
    graph = nx.DiGraph()
    graph.add_nodes_from(['a', 'b', 'c'])
    labels = ['1', '2', '3']
    up_nodes = [['a'], ['c', 'b'], ['a', 'b', 'c']]
    down_nodes = [['b'], ['a'], []]
    utils.paint_network_overtime_up_down(graph, list_up=up_nodes,
                                         list_down=down_nodes,
                                         color_down='blue',
                                         color_up='red',
                                         labels=labels,
                                         save_name='ok',
                                         create_gif=False)

"""


def test_delete_disconnected():
    g = nx.DiGraph()
    g.add_nodes_from(['a', 'b', 'c'])
    g.add_edge('a', 'b')
    large_g_connected = utils.delete_disconnected_network(g)
    large_g_connected.nodes()
    ok_(set(large_g_connected.nodes()) == {'b', 'a'})
    ok_(set(large_g_connected.edges()) == {('a', 'b')})


def test_add_attribute_to_network():
    g = nx.DiGraph()
    g.add_nodes_from(['a', 'b', 'c'])
    new_g = utils.add_attribute_to_network(g, ['a', 'b'], 'inTest', 'true')
    ok_(new_g.node['a'] == {'inTest': 'true'})
    ok_(new_g.node['c'] == {'inTest': 'false'})


def test_trim_source_sink():
    graph = nx.DiGraph()
    graph.add_edge('A', 'B')
    graph.add_edge('B', 'C')
    graph.add_edge('B', 'B')
    graph.add_edge('D', 'C')
    graph.add_edge('C', 'N')
    trimmed = utils.trim_sink_source_nodes(graph, ['A', 'B', 'D'],
                                           remove_self_edge=True)
    ok_(set(trimmed.edges()) == {('B', 'C'), ('A', 'B'), ('D', 'C')})
    return
    import matplotlib.pyplot as plt
    plt.subplot(121)
    nx.draw(graph,
            pos=nx.nx_pydot.graphviz_layout(graph, 'dot'),
            with_labels=True)
    plt.subplot(122)
    nx.draw(trimmed,
            pos=nx.nx_pydot.graphviz_layout(graph, 'dot'),
            with_labels=True)
    plt.show()

# test_trim_source_sink()



import os
import time

import matplotlib.image as mpimg
import matplotlib.pyplot as plt
import networkx as nx
import numpy as np
import pandas as pd
import requests

from py2cytoscape.data.cyrest_client import CyRestClient

IP = 'localhost'
PORT = 1234
VERSION = 'v1'

SUID_LIST = 'suid'
url = 'http://{}:{}/{}/'.format(IP, PORT, VERSION)


def create_point(value, lesser, equal, greater):
    return [{'value': str(value), 'lesser': lesser,
             'equal': equal, 'greater': greater}]


def create_slope(min_val=0, max_val=10, values=(1, 10)):
    point_1 = create_point(min_val, values[0], values[0], values[0])
    point_2 = create_point(max_val, values[1], values[1], values[1])
    return point_1 + point_2


class LayoutClient(object):
    def __init__(self):
        self.__url = url + 'apply/layouts'

    def get_all(self):
        return requests.get(self.__url).json()

    def get_option(self, name):
        _url = '{}/{}/columntypes'.format(self.__url, name)
        return requests.get(_url).json()

    def get_parameters(self, name):
        _url = '{}/{}/parameters'.format(self.__url, name)
        return requests.get(_url).json()

    def update(self, name, parameters='color'):
        _url = '{}/{}/{}'.format(self.__url, name, str(parameters))
        requests.get(_url)

    def apply(self, name='force-directed', network=None, params=None):
        self._check_net(network)
        _url = '{}/{}/{}'.format(self.__url, name, network.get_id())
        requests.get(_url, params)

    def bundle_edge(self, network=None):
        self._check_net(network)
        _url = '{}/edgebundling/{}'.format(self.__url, network.get_id())
        requests.get(_url)

    def fit(self, network):
        _url = self.__url + 'apply/fit/' + str(network.get_id())
        requests.get(_url)

    @staticmethod
    def _check_net(network):
        if network is None:
            raise ValueError('Target network is required')


class RenderModel(object):
    def __init__(self, graph, layout="attributes-layout", use_cyto_net=False):
        """

        Parameters
        ----------
        graph : nx.DiGraph
        layout : str
        use_cyto_net : bool
            Use already loaded network in cytoscape
        """

        self.graph = graph
        self.cy = CyRestClient()
        # self.cy.session.delete()
        self.cy.layout2 = LayoutClient()

        for n, (i, j) in enumerate(self.graph.edges()):
            self.graph[i][j]['name'] = '{},{}'.format(i, j)
        if use_cyto_net:
            self.g_cy = self.cy.network.create(self.cy.network.get_all()[0])
        else:
            self.g_cy = self.cy.network.create_from_networkx(self.graph)

        time.sleep(5)

        view_id_list = self.g_cy.get_views()
        self.view1 = self.g_cy.get_view(view_id_list[0], fmt='view')

        # Marquee Directed Simple
        self.style = self.cy.style.create('MAGINE')
        self.style.update_defaults(default_style)
        self.style.create_passthrough_mapping('name', vp='NODE_LABEL')
        if layout == 'attributes-layout':
            self.cy.layout2.apply(name=layout, network=self.g_cy,
                                  params={'column': 'color'})
        else:
            self.cy.layout2.apply(
                name=layout, network=self.g_cy,
                params={
                    'defaultSpringLength': 50,
                    'defaultNodeMass'    : 5,
                }
            )
        self.node_name2id = name2suid(self.g_cy, 'node')
        self.edge_name2id = name2suid(self.g_cy, 'edge')

    def print_options(self):
        """ print cytoscape options for network, style, nodes, edges

        :return:
        """
        vps = pd.Series(self.cy.style.vps.get_all())
        print(vps)
        node_vps = self.cy.style.vps.get_node_visual_props()
        for i in node_vps:
            print(i)
        print('\nNode options\n')
        edge_vps = self.cy.style.vps.get_edge_visual_props()
        for i in edge_vps:
            print(i)
        print('\nEdge options\n')
        network_vps = self.cy.style.vps.get_network_visual_props()
        for i in network_vps:
            print(i)
        style_opts = self.cy.style.get_all()
        print('\nStyle options\n')
        for i in style_opts:
            print(i)

    def visualize_by_list_of_time(self, list_of_time, labels=None,
                                  prefix='tmp', out_dir=None, scale=100):
        """
        create sequences of pdfs and svgs based on list of attributes

        list_of_time should point to attributes of the network.
        This attribute will update the network accordingly.

        Parameters
        ----------
        list_of_time : list_like
        labels : list_like
        prefix : str
        out_dir : str


        Returns
        -------

        """
        if out_dir is not None:
            if not os.path.exists(out_dir):
                os.mkdir(out_dir)
            if not os.path.exists(os.path.join(out_dir, 'Figures')):
                os.mkdir(os.path.join(out_dir, 'Figures'))

        edge_width = {}
        for i, j, d in self.graph.edges(data=True):
            edge_width[self.edge_name2id['{},{}'.format(i, j)]] = d['weight']

        _min, _max = min(edge_width.values()), max(edge_width.values())
        self.style.create_passthrough_mapping('name', vp='NODE_LABEL')
        self.style.create_continuous_mapping(
            column='weight', col_type='Double', vp='EDGE_WIDTH',
            points=create_slope(min_val=_min, max_val=_max, values=(3, 10))
        )

        self.cy.style.apply(style=self.style, network=self.g_cy)

        node_label_colors = {self.node_name2id[i]: 'black' for i in
                             self.graph.nodes}

        node_color_values = {self.node_name2id[i]: d['color'] for i, d in
                             self.graph.nodes(data=True)}
        # do all node changes
        df_vs_node = pd.DataFrame(
            [node_label_colors, node_color_values],
            index=['NODE_LABEL_COLOR', 'NODE_FILL_COLOR'],
        )

        df_vs_node = df_vs_node.T
        df_vs_node['NODE_BORDER_PAINT'] = df_vs_node['NODE_LABEL_COLOR']

        # do all edge changes
        edge_color_values = {}
        for i in self.edge_name2id:
            edge_color_values[self.edge_name2id[i]] = 'gray'

        df_edge = pd.DataFrame(index=self.edge_name2id.values())
        # df_edge = df_edge.T
        df_edge['EDGE_STROKE_UNSELECTED_PAINT'] = 'gray'
        df_edge['EDGE_SOURCE_ARROW_UNSELECTED_PAINT'] = 'gray'
        df_edge['EDGE_TARGET_ARROW_UNSELECTED_PAINT'] = 'gray'

        # update views
        self.view1.batch_update_node_views(df_vs_node)
        self.view1.batch_update_edge_views(df_edge)

        self.cy.layout.fit(network=self.g_cy)
        x = self.view1.get_network_view_as_dict()
        self.view1.update_network_view(
            visual_property='NETWORK_BACKGROUND_PAINT', value=u'#FFFFFF')
        self.view1.update_network_view(
            visual_property='NETWORK_SCALE_FACTOR',
            value=0.8 * x['NETWORK_SCALE_FACTOR']
        )

        x = self.view1.get_network_view_as_dict()
        self.create_png('out.png', 2400)
        trim_photo('out.png', 'x')

        all_node_size = []
        for ind, j in enumerate(list_of_time):
            size = np.array([self.graph.node[n]['sample{}'.format(j)]
                             for n in self.graph.nodes])
            all_node_size.append(size)
        size = np.array(all_node_size).flatten()
        simple_slope = create_slope(min_val=size.min(), max_val=size.max(),
                                    values=(10, scale))

        for j in sorted(list_of_time):
            time.sleep(1)
            self.style.create_continuous_mapping(column='sample{}'.format(j),
                                                 col_type='Double',
                                                 vp='NODE_SIZE',
                                                 points=simple_slope)
            self.cy.style.apply(style=self.style, network=self.g_cy)

            fig_name = '{0}_{1}'.format(prefix, j)
            print("Saving {}".format(fig_name))

            if out_dir is not None:
                out_file = os.path.join(out_dir, 'Figures',
                                        '{}.png'.format(fig_name))
            else:
                out_file = '{}.png'.format(fig_name)

            if os.path.exists(out_file):
                os.remove(out_file)

            x = self.view1.get_network_view_as_dict()
            self.create_png(out_file, int(x['NETWORK_WIDTH']))
            self.create_svg(out_file, int(x['NETWORK_WIDTH']))
            if labels is None:
                trim_photo(out_file, j)
            else:
                trim_photo(out_file, j)

    def update_node_color(self, attribute, save_name):
        self.cy.style.apply(style=self.style, network=self.g_cy)
        node_color_values = {self.node_name2id[i[0]]: i[1][attribute] for i in
                             self.graph.nodes(data=True)}

        self.view1.update_node_views(visual_property='NODE_FILL_COLOR',
                                     values=node_color_values)
        node_label_colors = {self.node_name2id[i]: 'black' for i in
                             self.graph.nodes}
        node_labels = {self.node_name2id[i]: i for i in self.graph.nodes()}
        node_color_values = {self.node_name2id[i]: d[attribute] for i, d in
                             self.graph.nodes(data=True)}
        # do all node changes
        df_vs_node = pd.DataFrame(
            [node_label_colors, node_color_values, node_labels],
            index=['NODE_LABEL_COLOR', 'NODE_FILL_COLOR', 'NODE_LABEL'],
        )

        df_vs_node = df_vs_node.T
        self.view1.batch_update_node_views(df_vs_node)
        with open('{0}.svg'.format(save_name), 'wb') as f:
            network_pdf = self.g_cy.get_svg()
            f.write(network_pdf)
            f.close()

        with open('{0}.pdf'.format(save_name), 'wb') as f:
            network_pdf = self.g_cy.get_pdf()
            f.write(network_pdf)
            f.close()

        with open('{0}.png'.format(save_name), 'wb') as f:
            network_pdf = self.g_cy.get_png()
            f.write(network_pdf)
            f.close()

    def create_png(self, out_file, width):
        with open(out_file, 'wb') as f:
            f.write(self.g_cy.get_png(width))

    def create_svg(self, out_file, width):
        with open(out_file.replace('png', 'svg'), 'wb') as f:
            f.write(self.get_svg(width))

    def get_svg(self, height=2000):
        url = '%sviews/first.svg?h=%d' % (self.g_cy._CyNetwork__url, height)
        return requests.get(url).content

    def color_nodes(self, nodes, color):
        self.reset_style()

        valid_nodes = {i for i in nodes if i in self.graph.nodes()}

        node_color_values = {self.node_name2id[i]: color for i in valid_nodes}
        node_label_colors = {self.node_name2id[i]: 'black' for i in
                             valid_nodes}
        node_size = {self.node_name2id[i]: 100 for i in valid_nodes}
        # do all node changes
        df_vs_node = pd.DataFrame(
            [node_color_values, node_size],
            index=['NODE_FILL_COLOR', 'NODE_SIZE']
        ).T

        self.view1.batch_update_node_views(df_vs_node)

    def reset_style(self):
        self.cy.style.apply(style=self.style, network=self.g_cy)


def name2suid(network, obj_type='node'):
    if obj_type is 'node':
        table = network.get_node_table()
    elif obj_type is 'edge':
        table = network.get_edge_table()
    else:
        raise ValueError('No such object type: ' + obj_type)
    table.reset_index(level=0, inplace=True)
    name_to_suid = {}
    for suid, name in table[['SUID', 'name']].values:
        name_to_suid[name] = suid
    return name_to_suid


def trim_photo(im_location, title=None):
    """
    Removes whitespace and adds title to image


    Parameters
    ----------
    im_location: str
        location of file, will be used for output as well
    title : str
        title to provide to add to image
        Will be the sample id

    Returns
    -------

    """

    img = mpimg.imread(im_location)
    mask = img[:, :, 0] < 1

    img = img[np.ix_(mask.any(1), mask.any(0))]

    plt.imshow(img, interpolation='none')
    plt.xticks([])
    plt.yticks([])
    if title is not None:
        plt.title(title, fontsize=24)
    plt.axis('off')
    # out = im_location.replace('.png', '_formatted.png')
    plt.savefig(im_location, dpi=2000, bbox_inches='tight', transparent=True)
    plt.close()


def trip_photo_old(im_location, title=None):
    """
    Removes whitespace and adds title to image


    Parameters
    ----------
    im_location: str
        location of file, will be used for output as well
    title : str
        title to provide to add to image
        Will be the sample id

    Returns
    -------

    """

    img = mpimg.imread(im_location)
    x_dim = img.shape[0]
    y_dim = img.shape[1]
    alpha = np.ones((x_dim, y_dim, 1))
    mask = (img[:, :, 0] == 1.) & (img[:, :, 1] == 1.)

    alpha[mask] = 0
    img = np.append(img, alpha, axis=2)
    to_remove_x = set()
    to_remove_y = set()

    for i in range(x_dim):
        if img[i, :, 0].sum() != y_dim:
            to_remove_x.add(i)
    for i in range(y_dim):
        if img[:, i, 0].sum() != x_dim:
            to_remove_y.add(i)
    if len(to_remove_x) > 2:
        _min, _max = min(to_remove_x), max(to_remove_x)
        _scale = int((.05 * x_dim))
        img = img[_min - _scale:_max + _scale, :, :]

    if len(to_remove_y) > 2:
        _min, _max = min(to_remove_y), max(to_remove_y)
        _scale = int((.05 * y_dim))
        img = img[:, _min - _scale:_max + _scale, :]

    plt.imshow(img, interpolation='none')
    plt.xticks([])
    plt.yticks([])
    if title is not None:
        plt.title(title)
    plt.axis('off')
    out = im_location.replace('.png', '_formatted.png')
    out2 = im_location.replace('.png', '_formatted.svg')
    plt.savefig(out, dpi=300, bbox_inches='tight', transparent=True)
    plt.savefig(out2, bbox_inches='tight', transparent=True)
    plt.close()
    # plt.show()


default_style = {
    u'EDGE_BEND'                         : None,
    u'EDGE_CURVED'                       : True,
    u'EDGE_LABEL'                        : u'',
    u'EDGE_LABEL_COLOR'                  : u'#000000',
    u'EDGE_LABEL_FONT_SIZE'              : 10,
    u'EDGE_LABEL_TRANSPARENCY'           : 255,
    u'EDGE_LABEL_WIDTH'                  : 200.0,
    u'EDGE_LINE_TYPE'                    : u'SOLID',
    u'EDGE_PAINT'                        : u'#808080',
    u'EDGE_SELECTED'                     : False,
    u'EDGE_SELECTED_PAINT'               : u'#FF0000',
    u'EDGE_SOURCE_ARROW_SELECTED_PAINT'  : u'#FFFF00',
    u'EDGE_SOURCE_ARROW_SHAPE'           : u'NONE',
    u'EDGE_SOURCE_ARROW_SIZE'            : 6.0,
    u'EDGE_SOURCE_ARROW_UNSELECTED_PAINT': u'#000000',
    u'EDGE_STROKE_SELECTED_PAINT'        : u'#FF0000',
    u'EDGE_STROKE_UNSELECTED_PAINT'      : u'#404040',
    u'EDGE_TARGET_ARROW_SELECTED_PAINT'  : u'#FFFF00',
    u'EDGE_TARGET_ARROW_SHAPE'           : u'Delta',
    u'EDGE_TARGET_ARROW_SIZE'            : 6.0,
    u'EDGE_TARGET_ARROW_UNSELECTED_PAINT': u'#000000',
    u'EDGE_TOOLTIP'                      : u'',
    u'EDGE_TRANSPARENCY'                 : 150,
    u'EDGE_UNSELECTED_PAINT'             : u'#404040',
    u'EDGE_VISIBLE'                      : True,
    u'EDGE_WIDTH'                        : 2.0,
    u'NETWORK_BACKGROUND_PAINT'          : u'#FFFFFF',
    u'NETWORK_CENTER_X_LOCATION'         : 0.0,
    u'NETWORK_CENTER_Y_LOCATION'         : 0.0,
    u'NETWORK_CENTER_Z_LOCATION'         : 0.0,
    u'NETWORK_DEPTH'                     : 0.0,
    u'NETWORK_EDGE_SELECTION'            : True,
    u'NETWORK_HEIGHT'                    : 400.0,
    u'NETWORK_NODE_SELECTION'            : True,
    u'NETWORK_SCALE_FACTOR': 1.0,
    u'NETWORK_SIZE': 550.0,
    u'NETWORK_TITLE': u'',
    u'NETWORK_WIDTH': 550.0,

    u'NODE_BORDER_PAINT': u'#000000',
    u'NODE_BORDER_STROKE': u'SOLID',
    u'NODE_BORDER_TRANSPARENCY': 255,
    u'NODE_BORDER_WIDTH': 2.0,
    u'NODE_DEPTH': 0.0,
    u'NODE_FILL_COLOR': u'LightGray',
    u'NODE_HEIGHT': 40.0,
    # u'NODE_LABEL': u'<passthroughMapping attributeName="name" ',
    u'NODE_LABEL_COLOR': u'black',
    u'NODE_LABEL_FONT_FACE': u'SansSerif.plain,plain,12',
    u'NODE_LABEL_FONT_SIZE': 24,
    u'NODE_LABEL_POSITION': u'C,C,c,0.50,0.00',
    u'NODE_LABEL_TRANSPARENCY': 255,
    u'NODE_LABEL_WIDTH': 200.0,
    u'NODE_NESTED_NETWORK_IMAGE_VISIBLE': True,
    u'NODE_PAINT': u'#787878',
    u'NODE_SELECTED': False,
    u'NODE_SELECTED_PAINT': u'#FFFF00',
    u'NODE_SHAPE': u'ELLIPSE',
    u'NODE_SIZE': 50.0,
    u'NODE_TOOLTIP': u'',
    u'NODE_TRANSPARENCY': 255,
    u'NODE_VISIBLE'                      : True,
    u'NODE_WIDTH'                        : 60.0,
    u'NODE_X_LOCATION'                   : 0.0,
    u'NODE_Y_LOCATION'                   : 0.0,
    u'NODE_Z_LOCATION'                   : 0.0
}

# default_style = {}
if __name__ == '__main__':
    # ddn = nx.nx.read_graphml('t_all_colored_pvalue_2.graphml')
    ddn = nx.DiGraph()
    ddn.add_node('x')
    ddn.add_edge('x', 'y')
    rm = RenderModel(ddn, style='Marquee')
    rm.print_options()
    quit()
    rm.visualize_by_list_of_time(
        ['time_0', 'time_1', 'time_2', 'time_3', 'time_4', ])



from __future__ import absolute_import
import logging
import platform
import socket
import time
import os
import warnings
from . import __version__

# code based off of magine.logging.py

SECONDS_IN_HOUR = 3600
LOG_LEVEL_ENV_VAR = 'MAGINE_LOG'
BASE_LOGGER_NAME = 'magine'
EXTENDED_DEBUG = 5
NAMED_LOG_LEVELS = {'NOTSET': logging.NOTSET,
                    'EXTENDED_DEBUG': EXTENDED_DEBUG,
                    'DEBUG': logging.DEBUG,
                    'INFO': logging.INFO,
                    'WARNING': logging.WARNING,
                    'ERROR': logging.ERROR,
                    'CRITICAL': logging.CRITICAL}


def formatter(time_utc=False):
    """
    Build a logging formatter using local or UTC time

    Parameters
    ----------
    time_utc : bool, optional (default: False)
        Use local time stamps in log messages if False, or Universal
        Coordinated Time (UTC) if True

    Returns
    -------
    A logging.Formatter object for MAGINE logging
    """
    log_fmt = logging.Formatter('%(asctime)s.%(msecs).3d - %(name)s - '
                                '%(levelname)s - %(message)s',
                                datefmt='%Y-%m-%d %H:%M:%S')
    if time_utc:
        log_fmt.converter = time.gmtime

    return log_fmt


def setup_logger(level=logging.INFO, console_output=True, file_output=False,
                 time_utc=False, capture_warnings=True):
    """
    Set up a new logging.Logger for MAGINE logging

    Calling this method will override any existing handlers, formatters etc.
    attached to the MAGINE logger. Typically, :func:`get_logger` should be
    used instead, which returns the existing MAGINE logger if it has already
    been set up, and can handle MAGINE submodule namespaces.

    Parameters
    ----------
    level : int
        Logging level, typically using a constant like logging.INFO or
        logging.DEBUG
    console_output : bool
        Set up a default console log handler if True (default)
    file_output : string
        Supply a filename to copy all log output to that file, or set to
        False to disable (default)
    time_utc : bool
        Specifies whether log entry time stamps should be in local time
        (when False) or UTC (True). See :func:`formatter` for more
        formatting options.
    capture_warnings : bool
        Capture warnings from Python's warnings module if True (default)

    Returns
    -------
    A logging.Logger object for MAGINE logging. Note that other MAGINE modules
    should use a logger specific to their namespace instead by calling
    :func:`get_logger`.
    """
    log = logging.getLogger(BASE_LOGGER_NAME)

    # Logging level can be overridden with environment variable
    if LOG_LEVEL_ENV_VAR in os.environ:
        try:
            level = int(os.environ[LOG_LEVEL_ENV_VAR])
        except ValueError:
            # Try parsing as a name
            level_name = os.environ[LOG_LEVEL_ENV_VAR]
            if level_name in NAMED_LOG_LEVELS.keys():
                level = NAMED_LOG_LEVELS[level_name]
            else:
                raise ValueError(
                    'Environment variable {} contains an invalid value "{}". '
                    'If set, its value must be one of {} (case-sensitive) or '
                    'an integer log level.'.format(
                        LOG_LEVEL_ENV_VAR, level_name,
                        ", ".join(NAMED_LOG_LEVELS.keys())
                    )
                )

    log.setLevel(level)

    # Remove default logging handler
    log.handlers = []

    log_fmt = formatter(time_utc=time_utc)

    if console_output:
        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(log_fmt)
        log.addHandler(stream_handler)

    if file_output:
        file_handler = logging.FileHandler(file_output)
        file_handler.setFormatter(log_fmt)
        log.addHandler(file_handler)

    log.info('Logging started on MAGINE version {}'.format(__version__))
    if time_utc:
        log.info('Log entry times are in UTC')
    else:
        utc_offset = time.timezone if (time.localtime().tm_isdst == 0) else \
            time.altzone
        utc_offset = -(utc_offset / SECONDS_IN_HOUR)
        log.info('Log entry time offset from UTC: %.2f hours', utc_offset)

    log.debug('OS Platform: %s', platform.platform())
    log.debug('Python version: %s', platform.python_version())
    log.debug('Hostname: %s', socket.getfqdn())

    logging.captureWarnings(capture_warnings)

    return log


def get_logger(logger_name=BASE_LOGGER_NAME, log_level=None,
               **kwargs):
    """
    Returns (if extant) or creates a MAGINE logger

    If the MAGINE base logger has already been set up, this method will return it
    or any of its descendant loggers without overriding the settings - i.e.
    any values supplied as kwargs will be ignored.

    Parameters
    ----------
    logger_name : string
        Get a logger for a specific namespace, typically __name__ for code
        outside of classes or self.__module__ inside a class
    log_level : bool or int
        Override the default or preset log level for the requested logger.
        None or False uses the default or preset value. True evaluates to
        logging.DEBUG. Any integer is used directly.
    **kwargs : kwargs
        Keyword arguments to supply to :func:`setup_logger`. Only used when
        the MAGINE logger hasn't been set up yet (i.e. there have been no
        calls to this function or :func:`get_logger` directly).

    Returns
    -------
    A logging.Logger object with the requested name

    Examples
    --------

    >>> from magine.logging import get_logger
    >>> logger = get_logger(__name__)
    >>> logger.debug('Test message')
    """
    if BASE_LOGGER_NAME not in logging.Logger.manager.loggerDict.keys():
        setup_logger(**kwargs)
    elif kwargs:
        warnings.warn('MAGINE logger already exists, ignoring keyword '
                      'arguments to setup_logger')

    logger = logging.getLogger(logger_name)

    if log_level is not None and log_level is not False:
        if isinstance(log_level, bool):
            log_level = logging.DEBUG
        elif not isinstance(log_level, int):
            raise ValueError('log_level must be a boolean, integer or None')

        if logger.getEffectiveLevel() != log_level:
            logger.debug('Changing log_level from %d to %d' % (
                logger.getEffectiveLevel(), log_level))
            logger.setLevel(log_level)
    return logger



import tempfile

from nose.tools import ok_

from magine.data.experimental_data import ExperimentalData
from magine.enrichment.enrichr import Enrichr, clean_drug_dbs, clean_tf_names, \
    get_background_list, get_libraries, run_enrichment_for_project
from magine.tests.sample_experimental_data import exp_data

e = Enrichr()


def test_single_run():
    e.print_valid_libs()
    list_2 = ['CASP3', 'CASP6', 'FAS', 'FADD', 'CASP8', 'CFLAR', 'BFAR', 'BAD',
              'BID', 'PMAIP1', 'MCL1', 'BCL2', 'BCL2L1', 'BAX', 'BAK1',
              'DIABLO', 'CYCS', 'PARP1', 'APAF1', 'XIAP']
    df = e.run(list_2, 'GO_Biological_Process_2017')
    terms = df['term_name']
    ok_(len(terms) == 185)


def test_libaries():
    get_libraries()


def test_project():
    slimmed = exp_data.species.copy()
    slimmed = slimmed.loc[slimmed.source.isin(['label_free', 'silac'])]
    slimmed = slimmed.loc[slimmed.sample_id.isin(['Time_1', 'Time_2',])]
    slimmed = ExperimentalData(slimmed)
    run_enrichment_for_project(slimmed, 'test',
                               databases=['KEGG_2016'])


def test_get_gene_set_lib():
    get_background_list('Phosphatase_Substrates_from_DEPOD')


def test_clean_drug_dbs():
    list_2 = ['CASP3', 'CASP6', 'FAS', 'FADD', 'CASP8', 'CFLAR', 'BFAR', 'BAD',
              'BID', 'PMAIP1', 'MCL1', 'BCL2', 'BCL2L1', 'BAX', 'BAK1',
              'DIABLO', 'CYCS', 'PARP1', 'APAF1', 'XIAP']
    df = e.run(list_2, ['Drug_Perturbations_from_GEO_2014',
                        'LINCS_L1000_Chem_Pert_up'
                        ])
    df = clean_drug_dbs(df)
    ok_(len(df['term_name']) == 6243)
    ok_(len(df.sig['term_name']) == 11)


def test_multi_sample():
    lists = [['BAX', 'BCL2', 'CASP3'],
             ['CASP10', 'CASP8', 'BAK'],
             ['BIM', 'CASP3']]
    df2 = e.run_samples(lists, ['1', '2', '3'], save_name='enrichr_test')
    ok_(df2.shape == (22, 11))


def test_multi_sample_plotting():
    up = exp_data.genes.sig.up_by_sample
    out_dir = tempfile.mkdtemp()
    e.run_samples(up, ['1', '2', '3'],
                  gene_set_lib=['Human_Phenotype_Ontology',
                            'MGI_Mammalian_Phenotype_2017'],
                  save_name='enrichr_test',
                  exp_data=exp_data,
                  create_html=True,
                  pivot=True,
                  out_dir=out_dir)


def test_set_of_dbs():
    lists = [['BAX', 'BCL2', 'CASP3'],
             ['CASP10', 'CASP8', 'BAK'],
             ['BIM', 'CASP3']]
    df2 = e.run_samples(lists, ['1', '2', '3'],
                        gene_set_lib=['KEGG_2016', 'NCI-Nature_2016'],
                        save_name='t')
    ok_(df2.shape == (60, 11))


def test_tf_names():
    df = e.run(['BAX', 'BCL2', 'MCL1'], ['ARCHS4_TFs_Coexp', 'ChEA_2016'])
    tfs = clean_tf_names(df)
    for i in tfs['term_name']:
        ok_('_' not in i)


if __name__ == '__main__':
    test_single_run()
